import datetime
from _typeshed import Incomplete, StrPath
from collections import defaultdict
from collections.abc import Callable, Iterable, Sequence
from contextlib import _GeneratorContextManager
from enum import IntEnum
from io import BytesIO
from typing import Any, ClassVar, NamedTuple, overload
from typing_extensions import Literal, TypeAlias

from PIL import Image

from .actions import Action
from .drawing import DrawingContext, PaintedPath
from .enums import (
    Align,
    AnnotationFlag,
    AnnotationName,
    Corner,
    FileAttachmentAnnotationName,
    PageLayout,
    PathPaintRule,
    RenderStyle,
    TextMarkupType,
    TextMode as TextMode,
    XPos as XPos,
    YPos as YPos,
)
from .recorder import FPDFRecorder
from .syntax import DestinationXYZ
from .util import _Unit

__all__ = ["FPDF", "XPos", "YPos", "get_page_format", "TextMode", "TitleStyle", "PAGE_FORMATS"]

_Orientation: TypeAlias = Literal["", "portrait", "p", "P", "landscape", "l", "L"]
_Format: TypeAlias = Literal["", "a3", "A3", "a4", "A4", "a5", "A5", "letter", "Letter", "legal", "Legal"]
_FontStyle: TypeAlias = Literal["", "B", "I"]
_FontStyles: TypeAlias = Literal["", "B", "I", "U", "BU", "UB", "BI", "IB", "IU", "UI", "BIU", "BUI", "IBU", "IUB", "UBI", "UIB"]
PAGE_FORMATS: dict[_Format, tuple[float, float]]

class DocumentState(IntEnum):
    UNINITIALIZED: int
    READY: int
    GENERATING_PAGE: int
    CLOSED: int

class Annotation(NamedTuple):
    type: str
    x: int
    y: int
    width: int
    height: int
    flags: tuple[AnnotationFlag, ...] = ...
    contents: str | None = ...
    link: str | int | None = ...
    alt_text: str | None = ...
    action: Action | None = ...
    color: int | None = ...
    modification_time: datetime.datetime | None = ...
    title: str | None = ...
    quad_points: Sequence[int] | None = ...
    page: int | None = ...
    border_width: int = ...
    name: AnnotationName | None = ...
    ink_list: tuple[int, ...] = ...
    embedded_file_name: str | None = ...
    field_type: str | None = ...
    value: str | None = ...
    def serialize(self, fpdf) -> str: ...

class EmbeddedFile(NamedTuple):
    basename: str
    bytes: bytes
    desc: str = ...
    creation_date: datetime.datetime | None = ...
    modification_date: datetime.datetime | None = ...
    compress: bool = ...
    checksum: bool = ...
    def file_spec(self, embedded_file_ref) -> str: ...

class TitleStyle(NamedTuple):
    font_family: str | None = ...
    font_style: str | None = ...
    font_size_pt: int | None = ...
    color: int | tuple[int, int, int] | None = ...
    underline: bool = ...
    t_margin: int | None = ...
    l_margin: int | None = ...
    b_margin: int | None = ...

class ToCPlaceholder(NamedTuple):
    render_function: Callable[[FPDF, Any], object]
    start_page: int
    y: int
    pages: int = ...

class SubsetMap:
    def __init__(self, identities: list[int]) -> None: ...
    def pick(self, unicode: int) -> int: ...
    def dict(self) -> dict[int, int]: ...

def get_page_format(format: _Format | tuple[float, float], k: float | None = ...) -> tuple[float, float]: ...

# TODO: TypedDicts
_Page: TypeAlias = dict[str, Any]
_Font: TypeAlias = dict[str, Any]
_FontFile: TypeAlias = dict[str, Any]
_Image: TypeAlias = dict[str, Any]

class FPDF:
    MARKDOWN_BOLD_MARKER: ClassVar[str]
    MARKDOWN_ITALICS_MARKER: ClassVar[str]
    MARKDOWN_UNDERLINE_MARKER: ClassVar[str]
    offsets: dict[int, int]
    page: int
    n: int
    buffer: bytearray
    pages: dict[int, _Page]
    state: DocumentState
    fonts: dict[str, _Font]
    font_files: dict[str, _FontFile]
    diffs: dict[int, int]
    images: dict[str, _Image]
    annots: defaultdict[int, list[Annotation]]
    links: dict[int, DestinationXYZ]
    embedded_files: list[Incomplete]
    embedded_files_per_pdf_ref: dict[Incomplete, Incomplete]
    in_footer: int
    lasth: int
    current_font: _Font
    font_family: str
    font_style: str
    font_stretching: float
    char_spacing: float
    underline: bool
    str_alias_nb_pages: str
    draw_color: str
    fill_color: str
    text_color: str
    page_background: Incomplete | None
    angle: int
    xmp_metadata: str | None
    image_filter: str
    page_duration: int
    page_transition: Incomplete | None
    struct_builder: Incomplete
    section_title_styles: Incomplete
    core_fonts: Incomplete
    core_fonts_encoding: str
    font_aliases: Incomplete
    k: float
    def_orientation: Incomplete
    font_size: float
    c_margin: float
    line_width: float
    dw_pt: float
    dh_pt: float
    pdf_version: str

    x: float
    y: float

    # Set during call to _set_orientation(), called from __init__().
    cur_orientation: Literal["P", "L"]
    w_pt: float
    h_pt: float
    w: float
    h: float
    def __init__(
        self,
        orientation: _Orientation = ...,
        unit: _Unit | float = ...,
        format: _Format | tuple[float, float] = ...,
        font_cache_dir: Literal["DEPRECATED"] = ...,
    ) -> None: ...
    @property
    def font_size_pt(self) -> float: ...
    @property
    def is_ttf_font(self) -> bool: ...
    @property
    def page_mode(self): ...
    @property
    def epw(self) -> float: ...
    @property
    def eph(self) -> float: ...
    @property
    def pages_count(self) -> int: ...
    def set_margin(self, margin: float) -> None: ...
    def set_margins(self, left: float, top: float, right: float = ...) -> None: ...
    l_margin: float
    def set_left_margin(self, margin: float) -> None: ...
    t_margin: float
    def set_top_margin(self, margin: float) -> None: ...
    r_margin: float
    def set_right_margin(self, margin: float) -> None: ...
    auto_page_break: bool
    b_margin: float
    page_break_trigger: float
    def set_auto_page_break(self, auto: bool, margin: float = ...) -> None: ...
    zoom_mode: Literal["fullpage", "fullwidth", "real", "default"] | float
    page_layout: PageLayout | None
    def set_display_mode(
        self,
        zoom: Literal["fullpage", "fullwidth", "real", "default"] | float,
        layout: Literal["single", "continuous", "two", "default"] = ...,
    ) -> None: ...
    compress: bool
    def set_compression(self, compress: bool) -> None: ...
    title: str
    def set_title(self, title: str) -> None: ...
    lang: str
    def set_lang(self, lang: str) -> None: ...
    subject: str
    def set_subject(self, subject: str) -> None: ...
    author: str
    def set_author(self, author: str) -> None: ...
    keywords: str
    def set_keywords(self, keywords: str) -> None: ...
    creator: str
    def set_creator(self, creator: str) -> None: ...
    producer: str
    def set_producer(self, producer: str) -> None: ...
    creation_date: datetime.datetime | bool | None
    def set_creation_date(self, date: datetime.datetime | bool | None = ...) -> None: ...
    def set_xmp_metadata(self, xmp_metadata: str) -> None: ...
    def set_doc_option(self, opt: str, value: str) -> None: ...
    def set_image_filter(self, image_filter: str) -> None: ...
    def alias_nb_pages(self, alias: str = ...) -> None: ...
    def open(self) -> None: ...
    def close(self) -> None: ...
    def add_page(
        self,
        orientation: _Orientation = ...,
        format: _Format | tuple[float, float] = ...,
        same: bool = ...,
        duration: int = ...,
        transition: Incomplete | None = ...,
    ) -> None: ...
    def header(self) -> None: ...
    def footer(self) -> None: ...
    def page_no(self) -> int: ...
    def set_draw_color(self, r: int, g: int = ..., b: int = ...) -> None: ...
    def set_fill_color(self, r: int, g: int = ..., b: int = ...) -> None: ...
    def set_text_color(self, r: int, g: int = ..., b: int = ...) -> None: ...
    def get_string_width(self, s: str, normalized: bool = ..., markdown: bool = ...) -> float: ...
    def set_line_width(self, width: float) -> None: ...
    def set_page_background(self, background) -> None: ...
    def drawing_context(self, debug_stream: Incomplete | None = ...) -> _GeneratorContextManager[DrawingContext]: ...
    def new_path(
        self, x: float = ..., y: float = ..., paint_rule: PathPaintRule = ..., debug_stream: Incomplete | None = ...
    ) -> _GeneratorContextManager[PaintedPath]: ...
    def draw_path(self, path: PaintedPath, debug_stream: Incomplete | None = ...) -> None: ...
    def set_dash_pattern(self, dash: float = ..., gap: float = ..., phase: float = ...) -> None: ...
    def line(self, x1: float, y1: float, x2: float, y2: float) -> None: ...
    def polyline(
        self, point_list: list[tuple[float, float]], fill: bool = ..., polygon: bool = ..., style: RenderStyle | str | None = ...
    ) -> None: ...
    def polygon(self, point_list: list[tuple[float, float]], fill: bool = ..., style: RenderStyle | str | None = ...) -> None: ...
    def dashed_line(self, x1, y1, x2, y2, dash_length: int = ..., space_length: int = ...) -> None: ...
    def rect(
        self,
        x: float,
        y: float,
        w: float,
        h: float,
        style: RenderStyle | str | None = ...,
        round_corners: tuple[str, ...] | tuple[Corner, ...] | bool = ...,
        corner_radius: float = ...,
    ) -> None: ...
    def ellipse(self, x: float, y: float, w: float, h: float, style: RenderStyle | str | None = ...) -> None: ...
    def circle(self, x: float, y: float, r, style: RenderStyle | str | None = ...) -> None: ...
    def regular_polygon(
        self,
        x: float,
        y: float,
        numSides: int,
        polyWidth: float,
        rotateDegrees: float = ...,
        style: RenderStyle | str | None = ...,
    ): ...
    def star(
        self,
        x: float,
        y: float,
        r_in: float,
        r_out: float,
        corners: int,
        rotate_degrees: float = ...,
        style: RenderStyle | str | None = ...,
    ): ...
    def arc(
        self,
        x: float,
        y: float,
        a: float,
        start_angle: float,
        end_angle: float,
        b: float | None = ...,
        inclination: float = ...,
        clockwise: bool = ...,
        start_from_center: bool = ...,
        end_at_center: bool = ...,
        style: RenderStyle | str | None = ...,
    ) -> None: ...
    def solid_arc(
        self,
        x: float,
        y: float,
        a: float,
        start_angle: float,
        end_angle: float,
        b: float | None = ...,
        inclination: float = ...,
        clockwise: bool = ...,
        style: RenderStyle | str | None = ...,
    ) -> None: ...
    def add_font(
        self, family: str, style: _FontStyle = ..., fname: str | None = ..., uni: bool | Literal["DEPRECATED"] = ...
    ) -> None: ...
    def set_font(self, family: str | None = ..., style: _FontStyles = ..., size: int = ...) -> None: ...
    def set_font_size(self, size: float) -> None: ...
    def set_char_spacing(self, spacing: float) -> None: ...
    def set_stretching(self, stretching: float) -> None: ...
    def add_link(self) -> int: ...
    def set_link(self, link, y: int = ..., x: int = ..., page: int = ..., zoom: float | Literal["null"] = ...) -> None: ...
    def link(
        self, x: float, y: float, w: float, h: float, link: str | int, alt_text: str | None = ..., border_width: int = ...
    ) -> Annotation: ...
    def embed_file(
        self,
        file_path: StrPath | None = ...,
        bytes: bytes | None = ...,
        basename: str | None = ...,
        modification_date: datetime.datetime | None = ...,
        *,
        creation_date: datetime.datetime | None = ...,
        desc: str = ...,
        compress: bool = ...,
        checksum: bool = ...,
    ) -> str: ...
    def file_attachment_annotation(
        self,
        file_path: StrPath,
        x: float,
        y: float,
        w: float = ...,
        h: float = ...,
        name: FileAttachmentAnnotationName | str | None = ...,
        flags: Iterable[AnnotationFlag | str] = ...,
        *,
        bytes: bytes | None = ...,
        basename: str | None = ...,
        creation_date: datetime.datetime | None = ...,
        modification_date: datetime.datetime | None = ...,
        desc: str = ...,
        compress: bool = ...,
        checksum: bool = ...,
    ) -> Annotation: ...
    def text_annotation(
        self,
        x: float,
        y: float,
        text: str,
        w: float = ...,
        h: float = ...,
        name: AnnotationName | str | None = ...,
        flags: tuple[AnnotationFlag, ...] | tuple[str, ...] = ...,
    ) -> None: ...
    def add_action(self, action, x: float, y: float, w: float, h: float) -> None: ...
    def highlight(
        self,
        text: str,
        title: str = ...,
        type: TextMarkupType | str = ...,
        color: tuple[float, float, float] = ...,
        modification_time: datetime.datetime | None = ...,
    ) -> _GeneratorContextManager[None]: ...
    add_highlight = highlight
    def add_text_markup_annotation(
        self,
        type: str,
        text: str,
        quad_points: Sequence[int],
        title: str = ...,
        color: tuple[float, float, float] = ...,
        modification_time: datetime.datetime | None = ...,
        page: int | None = ...,
    ) -> Annotation: ...
    def ink_annotation(
        self,
        coords: Iterable[Incomplete],
        contents: str = ...,
        title: str = ...,
        color: Sequence[float] = ...,
        border_width: int = ...,
    ) -> Annotation: ...
    def text(self, x: float, y: float, txt: str = ...) -> None: ...
    def rotate(self, angle: float, x: float | None = ..., y: float | None = ...) -> None: ...
    def rotation(self, angle: float, x: float | None = ..., y: float | None = ...) -> _GeneratorContextManager[None]: ...
    def local_context(
        self,
        font_family: Incomplete | None = ...,
        font_style: Incomplete | None = ...,
        font_size: Incomplete | None = ...,
        line_width: Incomplete | None = ...,
        draw_color: Incomplete | None = ...,
        fill_color: Incomplete | None = ...,
        text_color: Incomplete | None = ...,
        dash_pattern: Incomplete | None = ...,
        **kwargs,
    ) -> _GeneratorContextManager[None]: ...
    @property
    def accept_page_break(self) -> bool: ...
    def cell(
        self,
        w: float | None = ...,
        h: float | None = ...,
        txt: str = ...,
        border: bool | Literal[0, 1] | str = ...,
        ln: int | Literal["DEPRECATED"] = ...,
        align: str | Align = ...,
        fill: bool = ...,
        link: str = ...,
        center: bool | Literal["DEPRECATED"] = ...,
        markdown: bool = ...,
        new_x: XPos | str = ...,
        new_y: YPos | str = ...,
    ) -> bool: ...
    def will_page_break(self, height: float) -> bool: ...
    def multi_cell(
        self,
        w: float,
        h: float | None = ...,
        txt: str = ...,
        border: bool | Literal[0, 1] | str = ...,
        align: str | Align = ...,
        fill: bool = ...,
        split_only: bool = ...,
        link: str | int = ...,
        ln: int | Literal["DEPRECATED"] = ...,
        max_line_height: float | None = ...,
        markdown: bool = ...,
        print_sh: bool = ...,
        new_x: XPos | str = ...,
        new_y: YPos | str = ...,
    ): ...
    def write(self, h: float | None = ..., txt: str = ..., link: str = ..., print_sh: bool = ...) -> None: ...
    def image(
        self,
        name: str | Image.Image | BytesIO | StrPath,
        x: float | None = ...,
        y: float | None = ...,
        w: float = ...,
        h: float = ...,
        type: str = ...,
        link: str = ...,
        title: str | None = ...,
        alt_text: str | None = ...,
    ) -> _Image: ...
    def ln(self, h: float | None = ...) -> None: ...
    def get_x(self) -> float: ...
    def set_x(self, x: float) -> None: ...
    def get_y(self) -> float: ...
    def set_y(self, y: float) -> None: ...
    def set_xy(self, x: float, y: float) -> None: ...
    @overload
    def output(self, name: Literal[""] = ...) -> bytearray: ...  # type: ignore[misc]
    @overload
    def output(self, name: str) -> None: ...
    def normalize_text(self, txt: str) -> str: ...
    def sign_pkcs12(
        self,
        pkcs_filepath: str,
        password: bytes | None = ...,
        hashalgo: str = ...,
        contact_info: str | None = ...,
        location: str | None = ...,
        signing_time: datetime.datetime | None = ...,
        reason: str | None = ...,
        flags: tuple[AnnotationFlag, ...] = ...,
    ) -> None: ...
    def sign(
        self,
        key,
        cert,
        extra_certs: Sequence[Incomplete] = ...,
        hashalgo: str = ...,
        contact_info: str | None = ...,
        location: str | None = ...,
        signing_time: datetime.datetime | None = ...,
        reason: str | None = ...,
        flags: tuple[AnnotationFlag, ...] = ...,
    ) -> None: ...
    def file_id(self) -> str: ...
    def interleaved2of5(self, txt, x: float, y: float, w: float = ..., h: float = ...) -> None: ...
    def code39(self, txt, x: float, y: float, w: float = ..., h: float = ...) -> None: ...
    def rect_clip(self, x: float, y: float, w: float, h: float) -> _GeneratorContextManager[None]: ...
    def elliptic_clip(self, x: float, y: float, w: float, h: float) -> _GeneratorContextManager[None]: ...
    def round_clip(self, x: float, y: float, r: float) -> _GeneratorContextManager[None]: ...
    def unbreakable(self) -> _GeneratorContextManager[FPDFRecorder]: ...
    def offset_rendering(self) -> _GeneratorContextManager[FPDFRecorder]: ...
    def insert_toc_placeholder(self, render_toc_function, pages: int = ...) -> None: ...
    def set_section_title_styles(
        self,
        level0: TitleStyle,
        level1: TitleStyle | None = ...,
        level2: TitleStyle | None = ...,
        level3: TitleStyle | None = ...,
        level4: TitleStyle | None = ...,
        level5: TitleStyle | None = ...,
        level6: TitleStyle | None = ...,
    ) -> None: ...
    def start_section(self, name: str, level: int = ...) -> None: ...



from markdown.blockprocessors import BlockProcessor
from markdown.extensions import Extension

class MarkdownInHtmlProcessor(BlockProcessor): ...
class MarkdownInHtmlExtension(Extension): ...

def makeExtension(**kwargs): ...



from typing import Any

from . import _Serialiasable

KEYWORDS: Any
seq_types: Any

class Serialisable(_Serialiasable):
    __attrs__: Any
    __nested__: Any
    __elements__: Any
    __namespaced__: Any
    idx_base: int
    @property
    # TODO: needs overrides in many sub-classes
    # @abstractmethod
    def tagname(self) -> str: ...
    namespace: Any
    @classmethod
    def from_tree(cls, node): ...
    def to_tree(self, tagname: Any | None = ..., idx: Any | None = ..., namespace: Any | None = ...): ...
    def __iter__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self): ...
    def __add__(self, other): ...
    def __copy__(self): ...



import codecs
from typing_extensions import assert_type

assert_type(codecs.decode("x", "unicode-escape"), str)
assert_type(codecs.decode(b"x", "unicode-escape"), str)

assert_type(codecs.decode(b"x", "utf-8"), str)
codecs.decode("x", "utf-8")  # type: ignore

assert_type(codecs.decode("ab", "hex"), bytes)
assert_type(codecs.decode(b"ab", "hex"), bytes)



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class NamedStyle(Serialisable):  # type: ignore[misc]
    font: Any
    fill: Any
    border: Any
    alignment: Any
    number_format: Any
    protection: Any
    builtinId: Any
    hidden: Any
    # Overwritten by property below
    # xfId: Integer
    name: Any
    def __init__(
        self,
        name: str = ...,
        font=...,
        fill=...,
        border=...,
        alignment=...,
        number_format: Any | None = ...,
        protection=...,
        builtinId: Any | None = ...,
        hidden: bool = ...,
        xfId: Any | None = ...,
    ) -> None: ...
    def __setattr__(self, attr, value) -> None: ...
    def __iter__(self): ...
    @property
    def xfId(self): ...
    def bind(self, wb) -> None: ...
    def as_tuple(self): ...
    def as_xf(self): ...
    def as_name(self): ...

class NamedStyleList(list[Any]):
    @property
    def names(self): ...
    def __getitem__(self, key): ...
    def append(self, style) -> None: ...

class _NamedCellStyle(Serialisable):
    tagname: str
    name: Any
    xfId: Any
    builtinId: Any
    iLevel: Any
    hidden: Any
    customBuiltin: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        name: Any | None = ...,
        xfId: Any | None = ...,
        builtinId: Any | None = ...,
        iLevel: Any | None = ...,
        hidden: Any | None = ...,
        customBuiltin: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class _NamedCellStyleList(Serialisable):
    tagname: str
    # Overwritten by property below
    # count: Integer
    cellStyle: Any
    __attrs__: Any
    def __init__(self, count: Any | None = ..., cellStyle=...) -> None: ...
    @property
    def count(self): ...
    @property
    def names(self): ...



from typing import Any

from crontab import CronTab

class UserSpool(list[CronTab]):
    def __init__(self, loc: str, tabs: CronTabs | None = ...) -> None: ...
    def listdir(self, loc: str) -> list[str]: ...
    def get_owner(self, path: str) -> str: ...
    def generate(self, loc: str, username: str) -> CronTab: ...

class SystemTab(list[CronTab]):
    def __init__(self, loc: str, tabs: CronTabs | None = ...) -> None: ...

class AnaCronTab(list[CronTab]):
    def __init__(self, loc: str, tabs: CronTabs | None = ...) -> None: ...
    def add(self, loc: str, item: str, anajob) -> CronTab: ...

KNOWN_LOCATIONS: list[tuple[UserSpool | SystemTab | AnaCronTab, str]]

class CronTabs(list[UserSpool | SystemTab | AnaCronTab]):
    def __new__(cls, *args: Any, **kw: Any): ...
    def __init__(self) -> None: ...
    def add(self, cls: type[UserSpool | SystemTab | AnaCronTab], *args: Any) -> None: ...
    @property
    def all(self) -> CronTab: ...



import datetime
from typing import Any

NSP_COMPATIBLE: Any
MERIDIAN: Any
MICROSECOND: Any
EIGHT_DIGIT: Any
HOUR_MINUTE_REGEX: Any

def no_space_parser_eligibile(datestring): ...
def get_unresolved_attrs(parser_object): ...

date_order_chart: Any

def resolve_date_order(order, lst: Any | None = ...): ...

class _time_parser:
    time_directives: Any
    def __call__(self, timestring): ...

time_parser: Any

class _no_spaces_parser:
    period: Any
    date_formats: Any
    def __init__(self, *args, **kwargs): ...
    @classmethod
    def parse(cls, datestring, settings): ...

class _parser:
    alpha_directives: Any
    num_directives: Any
    settings: Any
    tokens: Any
    filtered_tokens: Any
    unset_tokens: Any
    day: Any
    month: Any
    year: Any
    time: Any
    auto_order: Any
    ordered_num_directives: Any
    def __init__(self, tokens, settings): ...
    @classmethod
    def parse(cls, datestring, settings, tz: datetime.tzinfo | None = ...): ...

class tokenizer:
    digits: str
    letters: str
    instream: Any
    def __init__(self, ds) -> None: ...
    def tokenize(self) -> None: ...



from typing import Any

from braintree.environment import Environment as Environment
from braintree.exceptions.configuration_error import ConfigurationError as ConfigurationError

class CredentialsParser:
    client_id: Any
    client_secret: Any
    access_token: Any
    def __init__(self, client_id: Any | None = ..., client_secret: Any | None = ..., access_token: Any | None = ...) -> None: ...
    environment: Any
    def parse_client_credentials(self) -> None: ...
    merchant_id: Any
    def parse_access_token(self) -> None: ...
    def get_environment(self, credential): ...
    def get_merchant_id(self, credential): ...



from typing import Any

from cryptography.hazmat.primitives.asymmetric.dsa import DSAPrivateKeyWithSerialization
from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKeyWithSerialization
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKeyWithSerialization
from cryptography.hazmat.primitives.serialization import KeySerializationEncryption
from cryptography.x509 import Certificate

def load_key_and_certificates(
    data: bytes, password: bytes | None, backend: Any | None = ...
) -> tuple[Any | None, Certificate | None, list[Certificate]]: ...
def serialize_key_and_certificates(
    name: bytes,
    key: RSAPrivateKeyWithSerialization | EllipticCurvePrivateKeyWithSerialization | DSAPrivateKeyWithSerialization,
    cert: Certificate | None,
    cas: list[Certificate] | None,
    encryption_algorithm: KeySerializationEncryption,
) -> bytes: ...



from _typeshed import Incomplete
from distutils.core import Distribution as _Distribution

from setuptools import SetuptoolsDeprecationWarning

class Distribution(_Distribution):
    def patch_missing_pkg_info(self, attrs) -> None: ...
    package_data: Incomplete
    dist_files: Incomplete
    src_root: Incomplete
    dependency_links: Incomplete
    setup_requires: Incomplete
    def __init__(self, attrs: Incomplete | None = ...) -> None: ...
    def warn_dash_deprecation(self, opt, section): ...
    def make_option_lowercase(self, opt, section): ...
    def parse_config_files(self, filenames: Incomplete | None = ..., ignore_option_errors: bool = ...) -> None: ...
    def fetch_build_eggs(self, requires): ...
    def finalize_options(self): ...
    def get_egg_cache_dir(self): ...
    def fetch_build_egg(self, req): ...
    def get_command_class(self, command): ...
    def print_commands(self): ...
    def get_command_list(self): ...
    def include(self, **attrs) -> None: ...
    packages: Incomplete
    py_modules: Incomplete
    ext_modules: Incomplete
    def exclude_package(self, package) -> None: ...
    def has_contents_for(self, package): ...
    def exclude(self, **attrs) -> None: ...
    def get_cmdline_options(self): ...
    def iter_distribution_names(self) -> None: ...
    def handle_display_options(self, option_order): ...

class DistDeprecationWarning(SetuptoolsDeprecationWarning): ...



from abc import abstractmethod
from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class AxId(Serialisable):  # type: ignore[misc]
    val: Any
    def __init__(self, val) -> None: ...

def PlotArea(): ...

class ChartBase(Serialisable):
    legend: Any
    layout: Any
    roundedCorners: Any
    axId: Any
    visible_cells_only: Any
    display_blanks: Any
    ser: Any
    series: Any
    title: Any
    anchor: str
    width: int
    height: float
    style: Any
    mime_type: str
    graphical_properties: Any
    __elements__: Any
    plot_area: Any
    pivotSource: Any
    pivotFormats: Any
    idx_base: int
    def __init__(self, axId=..., **kw) -> None: ...
    def __hash__(self): ...
    def __iadd__(self, other): ...
    def to_tree(self, namespace: Any | None = ..., tagname: Any | None = ..., idx: Any | None = ...): ...  # type: ignore[override]
    def set_categories(self, labels) -> None: ...
    def add_data(self, data, from_rows: bool = ..., titles_from_data: bool = ...) -> None: ...
    def append(self, value) -> None: ...
    @property
    def path(self): ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...



from _typeshed import Incomplete

import _win32typing

class error(Exception): ...

def ComparePath(path1: str, path2: str): ...
def CreateMDIFrame() -> _win32typing.PyCMDIFrameWnd: ...
def CreateMDIChild() -> _win32typing.PyCMDIChildWnd: ...
def CreateBitmap() -> _win32typing.PyCBitmap: ...
def CreateBitmapFromHandle(): ...
def CreateBrush() -> _win32typing.PyCBrush: ...
def CreateButton() -> _win32typing.PyCButton: ...
def CreateColorDialog(
    initColor: int = ..., flags: int = ..., parent: _win32typing.PyCWnd | None = ...
) -> _win32typing.PyCColorDialog: ...
def CreateControl(
    classId: str,
    windowName: str,
    style,
    rect: tuple[Incomplete, Incomplete, Incomplete, Incomplete],
    parent: _win32typing.PyCWnd,
    _id,
    bStorage,
    obPersist: Incomplete | None = ...,
    licKey: str | None = ...,
) -> _win32typing.PyCWnd: ...
def CreateControlBar() -> _win32typing.PyCControlBar: ...
def CreateCtrlView(doc: _win32typing.PyCDocument, className: str, style: int = ...) -> _win32typing.PyCCtrlView: ...
def CreateDC() -> None: ...
def CreateDCFromHandle(hwnd: int) -> _win32typing.PyCDC: ...
def CreateDialog(idRes, dll: _win32typing.PyDLL | None = ...) -> _win32typing.PyCDialog: ...
def CreateDialogBar() -> _win32typing.PyCDialogBar: ...
def CreateDialogIndirect(oblist) -> _win32typing.PyCDialog: ...
def CreatePrintDialog(
    idRes, bPrintSetupOnly, dwFlags, parent: _win32typing.PyCWnd | None = ..., dll: _win32typing.PyDLL | None = ...
) -> _win32typing.PyCPrintDialog: ...
def CreateDocTemplate(idRes) -> _win32typing.PyCDocTemplate: ...
def CreateEdit() -> _win32typing.PyCEdit: ...
def CreateFileDialog(
    bFileOpen,
    arg,
    defExt: str | None = ...,
    fileName: str | None = ...,
    _filter: str | None = ...,
    parent: _win32typing.PyCWnd | None = ...,
) -> _win32typing.PyCFileDialog: ...
def CreateFontDialog(
    arg, font: Incomplete | None = ..., dcPrinter: _win32typing.PyCDC | None = ..., parent: _win32typing.PyCWnd | None = ...
) -> _win32typing.PyCFontDialog: ...
def CreateFormView(doc: _win32typing.PyCDocument, Template) -> _win32typing.PyCFormView: ...
def CreateFrame(): ...
def CreateTreeCtrl() -> _win32typing.PyCTreeCtrl: ...
def CreateTreeView(doc: _win32typing.PyCDocument) -> _win32typing.PyCTreeView: ...
def CreatePalette(lp): ...
def CreatePopupMenu() -> _win32typing.PyCMenu: ...
def CreateMenu() -> _win32typing.PyCMenu: ...
def CreatePen(style, width, color): ...
def CreateProgressCtrl() -> _win32typing.PyCProgressCtrl: ...
def CreatePropertyPage(resource: _win32typing.PyResourceId, caption: int = ...) -> _win32typing.PyCPropertyPage: ...
def CreatePropertyPageIndirect(resourcelist: _win32typing.PyDialogTemplate, caption=...) -> _win32typing.PyCPropertyPage: ...
def CreatePropertySheet(
    caption: _win32typing.PyResourceId, parent: _win32typing.PyCWnd | None = ..., select=...
) -> _win32typing.PyCPropertySheet: ...
def CreateRgn() -> _win32typing.PyCRgn: ...
def CreateRichEditCtrl() -> _win32typing.PyCRichEditCtrl: ...
def CreateRichEditDocTemplate(idRes) -> _win32typing.PyCRichEditDocTemplate: ...
def CreateRichEditView(doc: _win32typing.PyCDocument | None = ...) -> _win32typing.PyCRichEditView: ...
def CreateSliderCtrl() -> _win32typing.PyCSliderCtrl: ...
def CreateSplitter() -> _win32typing.PyCSplitterWnd: ...
def CreateStatusBar(parent: _win32typing.PyCWnd, arg, arg1, ctrlStype=...) -> _win32typing.PyCStatusBar: ...
def CreateStatusBarCtrl() -> _win32typing.PyCStatusBarCtrl: ...
def CreateFont(properties) -> _win32typing.PyCFont: ...
def CreateToolBar(parent: _win32typing.PyCWnd, style, arg) -> _win32typing.PyCToolBar: ...
def CreateToolBarCtrl() -> _win32typing.PyCToolBarCtrl: ...
def CreateToolTipCtrl() -> _win32typing.PyCToolTipCtrl: ...
def CreateThread() -> _win32typing.PyCWinThread: ...
def CreateView(doc: _win32typing.PyCDocument) -> _win32typing.PyCScrollView: ...
def CreateEditView(doc: _win32typing.PyCDocument) -> _win32typing.PyCEditView: ...
def CreateDebuggerThread() -> None: ...
def CreateWindowFromHandle(hwnd: int) -> _win32typing.PyCWnd: ...
def CreateWnd() -> _win32typing.PyCWnd: ...
def DestroyDebuggerThread() -> None: ...
def DoWaitCursor(code) -> None: ...
def DisplayTraceback() -> None: ...
def Enable3dControls(): ...
def FindWindow(className: str, windowName: str) -> _win32typing.PyCWnd: ...
def FindWindowEx(
    parentWindow: _win32typing.PyCWnd, childAfter: _win32typing.PyCWnd, className: str, windowName: str
) -> _win32typing.PyCWnd: ...
def FullPath(path: str) -> str: ...
def GetActiveWindow() -> _win32typing.PyCWnd: ...
def GetApp() -> _win32typing.PyCWinApp: ...
def GetAppName(): ...
def GetAppRegistryKey() -> None: ...
def GetBytes(address, size) -> str: ...
def GetCommandLine() -> str: ...
def GetDeviceCaps(hdc, index): ...
def GetFileTitle(fileName: str) -> str: ...
def GetFocus() -> _win32typing.PyCWnd: ...
def GetForegroundWindow() -> _win32typing.PyCWnd: ...
def GetHalftoneBrush() -> _win32typing.PyCBrush: ...
def GetInitialStateRequest(): ...
def GetMainFrame() -> _win32typing.PyCWnd: ...
def GetName() -> str: ...
def GetProfileFileName() -> str: ...
def GetProfileVal(section: str, entry: str, defValue: str) -> str: ...
def GetResource() -> _win32typing.PyDLL: ...
def GetThread() -> _win32typing.PyCWinApp: ...
def GetType(): ...
def InitRichEdit() -> str: ...
def InstallCallbackCaller(): ...
def IsDebug() -> int: ...
def IsWin32s() -> int: ...
def IsObject(__o: object) -> bool: ...
def LoadDialogResource(idRes, dll: _win32typing.PyDLL | None = ...): ...
def LoadLibrary(fileName: str) -> _win32typing.PyDLL: ...
def LoadMenu(_id, dll: _win32typing.PyDLL | None = ...) -> _win32typing.PyCMenu: ...
def LoadStdProfileSettings(maxFiles) -> None: ...
def LoadString(stringId) -> str: ...
def MessageBox(message: str, arg, title: str | None = ...): ...
def OutputDebugString(msg: str) -> None: ...
def EnableControlContainer(): ...
def PrintTraceback(tb, output) -> None: ...
def PumpWaitingMessages(firstMessage, lastMessage): ...
def RegisterWndClass(style, hCursor: int = ..., hBrush: int = ..., hIcon=...) -> str: ...
def RemoveRecentFile(index: int = ...) -> None: ...
def SetAppHelpPath(): ...
def SetAppName(appName: str): ...
def SetCurrentInstanceHandle(newVal): ...
def SetCurrentResourceHandle(newVal): ...
def SetDialogBkColor(arg, arg1): ...
def SetProfileFileName(filename: str) -> None: ...
def SetRegistryKey(key: str) -> None: ...
def SetResource(dll) -> _win32typing.PyDLL: ...
def SetStatusText(msg: str, bForce: int = ...) -> None: ...
def StartDebuggerPump() -> None: ...
def StopDebuggerPump() -> None: ...
def TranslateMessage(): ...
def TranslateVirtualKey(vk) -> str: ...
def WinHelp(arg, data: str) -> None: ...
def WriteProfileVal(section: str, entry: str, value: str) -> None: ...
def AddToRecentFileList(*args, **kwargs): ...  # incomplete
def CreateImageList(*args, **kwargs): ...  # incomplete
def CreateListCtrl(*args, **kwargs): ...  # incomplete
def CreateListView(*args, **kwargs): ...  # incomplete
def CreateRectRgn(*args, **kwargs): ...  # incomplete
def GetRecentFileList(*args, **kwargs): ...  # incomplete
def OutputDebug(*args, **kwargs): ...  # incomplete

AFX_IDW_PANE_FIRST: int
AFX_IDW_PANE_LAST: int
AFX_WS_DEFAULT_VIEW: int
CDocTemplate_Confidence_maybeAttemptForeign: int
CDocTemplate_Confidence_maybeAttemptNative: int
CDocTemplate_Confidence_noAttempt: int
CDocTemplate_Confidence_yesAlreadyOpen: int
CDocTemplate_Confidence_yesAttemptForeign: int
CDocTemplate_Confidence_yesAttemptNative: int
CDocTemplate_docName: int
CDocTemplate_fileNewName: int
CDocTemplate_filterExt: int
CDocTemplate_filterName: int
CDocTemplate_regFileTypeId: int
CDocTemplate_regFileTypeName: int
CDocTemplate_windowTitle: int
CRichEditView_WrapNone: int
CRichEditView_WrapToTargetDevice: int
CRichEditView_WrapToWindow: int
debug: int
FWS_ADDTOTITLE: int
FWS_PREFIXTITLE: int
FWS_SNAPTOBARS: int
ID_APP_ABOUT: int
ID_APP_EXIT: int
ID_EDIT_CLEAR: int
ID_EDIT_CLEAR_ALL: int
ID_EDIT_COPY: int
ID_EDIT_CUT: int
ID_EDIT_FIND: int
ID_EDIT_GOTO_LINE: int
ID_EDIT_PASTE: int
ID_EDIT_REDO: int
ID_EDIT_REPEAT: int
ID_EDIT_REPLACE: int
ID_EDIT_SELECT_ALL: int
ID_EDIT_SELECT_BLOCK: int
ID_EDIT_UNDO: int
ID_FILE_CHECK: int
ID_FILE_CLOSE: int
ID_FILE_IMPORT: int
ID_FILE_LOCATE: int
ID_FILE_MRU_FILE1: int
ID_FILE_MRU_FILE2: int
ID_FILE_MRU_FILE3: int
ID_FILE_MRU_FILE4: int
ID_FILE_NEW: int
ID_FILE_OPEN: int
ID_FILE_PAGE_SETUP: int
ID_FILE_PRINT: int
ID_FILE_PRINT_PREVIEW: int
ID_FILE_PRINT_SETUP: int
ID_FILE_RUN: int
ID_FILE_SAVE: int
ID_FILE_SAVE_ALL: int
ID_FILE_SAVE_AS: int
ID_HELP_GUI_REF: int
ID_HELP_OTHER: int
ID_HELP_PYTHON: int
ID_INDICATOR_COLNUM: int
ID_INDICATOR_LINENUM: int
ID_NEXT_PANE: int
ID_PREV_PANE: int
ID_SEPARATOR: int
ID_VIEW_BROWSE: int
ID_VIEW_EOL: int
ID_VIEW_FIXED_FONT: int
ID_VIEW_FOLD_COLLAPSE: int
ID_VIEW_FOLD_COLLAPSE_ALL: int
ID_VIEW_FOLD_EXPAND: int
ID_VIEW_FOLD_EXPAND_ALL: int
ID_VIEW_INDENTATIONGUIDES: int
ID_VIEW_INTERACTIVE: int
ID_VIEW_OPTIONS: int
ID_VIEW_RIGHT_EDGE: int
ID_VIEW_STATUS_BAR: int
ID_VIEW_TOOLBAR: int
ID_VIEW_TOOLBAR_DBG: int
ID_VIEW_WHITESPACE: int
ID_WINDOW_ARRANGE: int
ID_WINDOW_CASCADE: int
ID_WINDOW_NEW: int
ID_WINDOW_SPLIT: int
ID_WINDOW_TILE_HORZ: int
ID_WINDOW_TILE_VERT: int
IDB_BROWSER_HIER: int
IDB_DEBUGGER_HIER: int
IDB_HIERFOLDERS: int
IDC_ABOUT_VERSION: int
IDC_AUTO_RELOAD: int
IDC_AUTOCOMPLETE: int
IDC_BUTTON1: int
IDC_BUTTON2: int
IDC_BUTTON3: int
IDC_BUTTON4: int
IDC_CALLTIPS: int
IDC_CHECK1: int
IDC_CHECK2: int
IDC_CHECK3: int
IDC_COMBO1: int
IDC_COMBO2: int
IDC_EDIT1: int
IDC_EDIT2: int
IDC_EDIT3: int
IDC_EDIT4: int
IDC_EDIT_TABS: int
IDC_INDENT_SIZE: int
IDC_KEYBOARD_CONFIG: int
IDC_PROMPT1: int
IDC_PROMPT2: int
IDC_PROMPT3: int
IDC_PROMPT4: int
IDC_PROMPT_TABS: int
IDC_RADIO1: int
IDC_RADIO2: int
IDC_RIGHTEDGE_COLUMN: int
IDC_RIGHTEDGE_DEFINE: int
IDC_RIGHTEDGE_ENABLE: int
IDC_RIGHTEDGE_SAMPLE: int
IDC_SPIN1: int
IDC_SPIN2: int
IDC_SPIN3: int
IDC_TAB_SIZE: int
IDC_USE_SMART_TABS: int
IDC_USE_TABS: int
IDC_VIEW_WHITESPACE: int
IDC_VSS_INTEGRATE: int
IDD_ABOUTBOX: int
IDD_DUMMYPROPPAGE: int
IDD_GENERAL_STATUS: int
IDD_LARGE_EDIT: int
IDD_PP_DEBUGGER: int
IDD_PP_EDITOR: int
IDD_PP_FORMAT: int
IDD_PP_IDE: int
IDD_PP_TABS: int
IDD_PP_TOOLMENU: int
IDD_PROPDEMO1: int
IDD_PROPDEMO2: int
IDD_RUN_SCRIPT: int
IDD_SET_TABSTOPS: int
IDD_SIMPLE_INPUT: int
IDD_TREE: int
IDD_TREE_MB: int
IDR_CNTR_INPLACE: int
IDR_DEBUGGER: int
IDR_MAINFRAME: int
IDR_PYTHONCONTYPE: int
IDR_PYTHONTYPE: int
IDR_PYTHONTYPE_CNTR_IP: int
IDR_TEXTTYPE: int
LM_COMMIT: int
LM_HORZ: int
LM_HORZDOCK: int
LM_LENGTHY: int
LM_MRUWIDTH: int
LM_STRETCH: int
LM_VERTDOCK: int
MFS_4THICKFRAME: int
MFS_BLOCKSYSMENU: int
MFS_MOVEFRAME: int
MFS_SYNCACTIVE: int
MFS_THICKFRAME: int
PD_ALLPAGES: int
PD_COLLATE: int
PD_DISABLEPRINTTOFILE: int
PD_ENABLEPRINTHOOK: int
PD_ENABLEPRINTTEMPLATE: int
PD_ENABLEPRINTTEMPLATEHANDLE: int
PD_ENABLESETUPHOOK: int
PD_ENABLESETUPTEMPLATE: int
PD_ENABLESETUPTEMPLATEHANDLE: int
PD_HIDEPRINTTOFILE: int
PD_NONETWORKBUTTON: int
PD_NOPAGENUMS: int
PD_NOSELECTION: int
PD_NOWARNING: int
PD_PAGENUMS: int
PD_PRINTSETUP: int
PD_PRINTTOFILE: int
PD_RETURNDC: int
PD_RETURNDEFAULT: int
PD_RETURNIC: int
PD_SELECTION: int
PD_SHOWHELP: int
PD_USEDEVMODECOPIES: int
PD_USEDEVMODECOPIESANDCOLLATE: int
PSWIZB_BACK: int
PSWIZB_DISABLEDFINISH: int
PSWIZB_FINISH: int
PSWIZB_NEXT: int
IDC_DBG_ADD: int
IDC_DBG_BREAKPOINTS: int
IDC_DBG_CLEAR: int
IDC_DBG_CLOSE: int
IDC_DBG_GO: int
IDC_DBG_STACK: int
IDC_DBG_STEP: int
IDC_DBG_STEPOUT: int
IDC_DBG_STEPOVER: int
IDC_DBG_WATCH: int
IDC_EDITOR_COLOR: int
IDC_FOLD_ENABLE: int
IDC_FOLD_ON_OPEN: int
IDC_FOLD_SHOW_LINES: int
IDC_LIST1: int
IDC_MARGIN_FOLD: int
IDC_MARGIN_LINENUMBER: int
IDC_MARGIN_MARKER: int
IDC_TABTIMMY_BG: int
IDC_TABTIMMY_IND: int
IDC_TABTIMMY_NONE: int
IDC_VIEW_EOL: int
IDC_VIEW_INDENTATIONGUIDES: int
ID_VIEW_FOLD_TOPLEVEL: int
UNICODE: int
copyright: str
dllhandle: int
types: dict[str, type]



from collections.abc import Callable
from typing import Any, TypeVar, overload

# connection and cursor not available at runtime
from psycopg2._psycopg import (
    BINARY as BINARY,
    DATETIME as DATETIME,
    NUMBER as NUMBER,
    ROWID as ROWID,
    STRING as STRING,
    Binary as Binary,
    DatabaseError as DatabaseError,
    DataError as DataError,
    Date as Date,
    DateFromTicks as DateFromTicks,
    Error as Error,
    IntegrityError as IntegrityError,
    InterfaceError as InterfaceError,
    InternalError as InternalError,
    NotSupportedError as NotSupportedError,
    OperationalError as OperationalError,
    ProgrammingError as ProgrammingError,
    Time as Time,
    TimeFromTicks as TimeFromTicks,
    Timestamp as Timestamp,
    TimestampFromTicks as TimestampFromTicks,
    Warning as Warning,
    __libpq_version__ as __libpq_version__,
    apilevel as apilevel,
    connection as connection,
    cursor as cursor,
    paramstyle as paramstyle,
    threadsafety as threadsafety,
)

_T_conn = TypeVar("_T_conn", bound=connection)

@overload
def connect(dsn: str, connection_factory: Callable[..., _T_conn], cursor_factory: None = ..., **kwargs: Any) -> _T_conn: ...
@overload
def connect(
    dsn: str | None = ..., *, connection_factory: Callable[..., _T_conn], cursor_factory: None = ..., **kwargs: Any
) -> _T_conn: ...
@overload
def connect(
    dsn: str | None = ...,
    connection_factory: Callable[..., connection] | None = ...,
    cursor_factory: Callable[..., cursor] | None = ...,
    **kwargs: Any,
) -> connection: ...



from abc import abstractmethod
from typing import Any

from ._3d import _3DBase
from ._chart import ChartBase

class _BarChartBase(ChartBase):
    barDir: Any
    type: Any
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    dataLabels: Any
    __elements__: Any
    def __init__(
        self, barDir: str = ..., grouping: str = ..., varyColors: Any | None = ..., ser=..., dLbls: Any | None = ..., **kw
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class BarChart(_BarChartBase):
    tagname: str
    barDir: Any
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    gapWidth: Any
    overlap: Any
    serLines: Any
    extLst: Any
    x_axis: Any
    y_axis: Any
    __elements__: Any
    legend: Any
    def __init__(
        self, gapWidth: int = ..., overlap: Any | None = ..., serLines: Any | None = ..., extLst: Any | None = ..., **kw
    ) -> None: ...

class BarChart3D(_BarChartBase, _3DBase):
    tagname: str
    barDir: Any
    grouping: Any
    varyColors: Any
    ser: Any
    dLbls: Any
    view3D: Any
    floor: Any
    sideWall: Any
    backWall: Any
    gapWidth: Any
    gapDepth: Any
    shape: Any
    serLines: Any
    extLst: Any
    x_axis: Any
    y_axis: Any
    z_axis: Any
    __elements__: Any
    def __init__(
        self,
        gapWidth: int = ...,
        gapDepth: int = ...,
        shape: Any | None = ...,
        serLines: Any | None = ...,
        extLst: Any | None = ...,
        **kw,
    ) -> None: ...



from collections.abc import Iterable, Sequence
from socket import socket
from typing import Any

from .compat import HAS_IPV6 as HAS_IPV6, PY2 as PY2, WIN as WIN, string_types as string_types
from .proxy_headers import PROXY_HEADERS as PROXY_HEADERS

truthy: frozenset[Any]
KNOWN_PROXY_HEADERS: frozenset[Any]

def asbool(s: bool | str | int | None) -> bool: ...
def asoctal(s: str) -> int: ...
def aslist_cronly(value: str) -> list[str]: ...
def aslist(value: str) -> list[str]: ...
def asset(value: str | None) -> set[str]: ...
def slash_fixed_str(s: str | None) -> str: ...
def str_iftruthy(s: str | None) -> str | None: ...
def as_socket_list(sockets: Sequence[object]) -> list[socket]: ...

class _str_marker(str): ...
class _int_marker(int): ...
class _bool_marker: ...

class Adjustments:
    host: _str_marker = ...
    port: _int_marker = ...
    listen: list[str] = ...
    threads: int = ...
    trusted_proxy: str | None = ...
    trusted_proxy_count: int | None = ...
    trusted_proxy_headers: set[str] = ...
    log_untrusted_proxy_headers: bool = ...
    clear_untrusted_proxy_headers: _bool_marker | bool = ...
    url_scheme: str = ...
    url_prefix: str = ...
    ident: str = ...
    backlog: int = ...
    recv_bytes: int = ...
    send_bytes: int = ...
    outbuf_overflow: int = ...
    outbuf_high_watermark: int = ...
    inbuf_overflow: int = ...
    connection_limit: int = ...
    cleanup_interval: int = ...
    channel_timeout: int = ...
    log_socket_errors: bool = ...
    max_request_header_size: int = ...
    max_request_body_size: int = ...
    expose_tracebacks: bool = ...
    unix_socket: str | None = ...
    unix_socket_perms: int = ...
    socket_options: list[tuple[int, int, int]] = ...
    asyncore_loop_timeout: int = ...
    asyncore_use_poll: bool = ...
    ipv4: bool = ...
    ipv6: bool = ...
    sockets: list[socket] = ...
    def __init__(self, **kw: Any) -> None: ...
    @classmethod
    def parse_args(cls, argv: str) -> tuple[dict[str, Any], Any]: ...
    @classmethod
    def check_sockets(cls, sockets: Iterable[socket]) -> None: ...



import builtins as builtins
from collections.abc import Iterable, Sequence
from io import BytesIO as BytesIO, StringIO as StringIO
from typing import Any, TypeVar

_T = TypeVar("_T")

PY2: bool

string_types: type[Any] | Sequence[type[Any]]
text_type: type[Any] | Sequence[type[Any]]
bytes_types: type[Any] | Sequence[type[Any]]
integer_types: type[Any] | Sequence[type[Any]]
long = int

def input(prompt: Any) -> str: ...
def decodebytes(s: bytes) -> bytes: ...
def encodebytes(s: bytes) -> bytes: ...

bytes = builtins.bytes

def byte_ord(c: int | str) -> int: ...
def byte_chr(c: int) -> bytes: ...
def byte_mask(c: int, mask: int) -> bytes: ...
def b(s: bytes | str, encoding: str = ...) -> bytes: ...
def u(s: bytes | str, encoding: str = ...) -> str: ...
def b2s(s: bytes | str) -> str: ...
def is_callable(c: Any) -> bool: ...
def next(c: Iterable[_T]) -> _T: ...

MAXSIZE: int



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol Buffers - Google's data interchange format
Copyright 2008 Google Inc.  All rights reserved.
https://developers.google.com/protocol-buffers/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimer
in the documentation and/or other materials provided with the
distribution.
    * Neither the name of Google Inc. nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.internal.well_known_types
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _NullValue:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _NullValueEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_NullValue.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NULL_VALUE: _NullValue.ValueType  # 0
    """Null value."""

class NullValue(_NullValue, metaclass=_NullValueEnumTypeWrapper):
    """`NullValue` is a singleton enumeration to represent the null value for the
    `Value` type union.

     The JSON representation for `NullValue` is JSON `null`.
    """

NULL_VALUE: NullValue.ValueType  # 0
"""Null value."""
global___NullValue = NullValue

@typing_extensions.final
class Struct(google.protobuf.message.Message, google.protobuf.internal.well_known_types.Struct):
    """`Struct` represents a structured data value, consisting of fields
    which map to dynamically typed values. In some languages, `Struct`
    might be supported by a native representation. For example, in
    scripting languages like JS a struct is represented as an
    object. The details of that representation are described together
    with the proto support for the language.

    The JSON representation for `Struct` is JSON object.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class FieldsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str | None = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    FIELDS_FIELD_NUMBER: builtins.int
    @property
    def fields(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]:
        """Unordered map of dynamically typed values."""
    def __init__(
        self,
        *,
        fields: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["fields", b"fields"]) -> None: ...

global___Struct = Struct

@typing_extensions.final
class Value(google.protobuf.message.Message):
    """`Value` represents a dynamically typed value which can be either
    null, a number, a string, a boolean, a recursive struct value, or a
    list of values. A producer of value is expected to set one of these
    variants. Absence of any variant indicates an error.

    The JSON representation for `Value` is JSON value.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NULL_VALUE_FIELD_NUMBER: builtins.int
    NUMBER_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    BOOL_VALUE_FIELD_NUMBER: builtins.int
    STRUCT_VALUE_FIELD_NUMBER: builtins.int
    LIST_VALUE_FIELD_NUMBER: builtins.int
    null_value: global___NullValue.ValueType
    """Represents a null value."""
    number_value: builtins.float
    """Represents a double value."""
    string_value: builtins.str
    """Represents a string value."""
    bool_value: builtins.bool
    """Represents a boolean value."""
    @property
    def struct_value(self) -> global___Struct:
        """Represents a structured value."""
    @property
    def list_value(self) -> global___ListValue:
        """Represents a repeated `Value`."""
    def __init__(
        self,
        *,
        null_value: global___NullValue.ValueType | None = ...,
        number_value: builtins.float | None = ...,
        string_value: builtins.str | None = ...,
        bool_value: builtins.bool | None = ...,
        struct_value: global___Struct | None = ...,
        list_value: global___ListValue | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bool_value", b"bool_value", "kind", b"kind", "list_value", b"list_value", "null_value", b"null_value", "number_value", b"number_value", "string_value", b"string_value", "struct_value", b"struct_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bool_value", b"bool_value", "kind", b"kind", "list_value", b"list_value", "null_value", b"null_value", "number_value", b"number_value", "string_value", b"string_value", "struct_value", b"struct_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["kind", b"kind"]) -> typing_extensions.Literal["null_value", "number_value", "string_value", "bool_value", "struct_value", "list_value"] | None: ...

global___Value = Value

@typing_extensions.final
class ListValue(google.protobuf.message.Message, google.protobuf.internal.well_known_types.ListValue):
    """`ListValue` is a wrapper around a repeated field of values.

    The JSON representation for `ListValue` is JSON array.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Value]:
        """Repeated field of dynamically typed values."""
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___Value] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["values", b"values"]) -> None: ...

global___ListValue = ListValue



class SlumberBaseException(Exception): ...

class SlumberHttpBaseException(SlumberBaseException):
    def __init__(self, *args, **kwargs) -> None: ...

class HttpClientError(SlumberHttpBaseException): ...
class HttpNotFoundError(HttpClientError): ...
class HttpServerError(SlumberHttpBaseException): ...
class SerializerNoAvailable(SlumberBaseException): ...
class SerializerNotAvailable(SlumberBaseException): ...
class ImproperlyConfigured(SlumberBaseException): ...



#!/usr/bin/env python3

"""Script to generate unannotated baseline stubs using stubgen.

Basic usage:
$ python3 scripts/create_baseline_stubs.py <project on PyPI>

Run with -h for more help.
"""

from __future__ import annotations

import argparse
import os
import re
import subprocess
import sys

if sys.version_info >= (3, 8):
    from importlib.metadata import distribution

PYRIGHT_CONFIG = "pyrightconfig.stricter.json"


def search_pip_freeze_output(project: str, output: str) -> tuple[str, str] | None:
    # Look for lines such as "typed-ast==1.4.2".  '-' matches '_' and
    # '_' matches '-' in project name, so that "typed_ast" matches
    # "typed-ast", and vice versa.
    regex = "^(" + re.sub(r"[-_]", "[-_]", project) + ")==(.*)"
    m = re.search(regex, output, flags=re.IGNORECASE | re.MULTILINE)
    if not m:
        return None
    return m.group(1), m.group(2)


def get_installed_package_info(project: str) -> tuple[str, str] | None:
    """Find package information from pip freeze output.

    Match project name somewhat fuzzily (case sensitive; '-' matches '_', and
    vice versa).

    Return (normalized project name, installed version) if successful.
    """
    r = subprocess.run(["pip", "freeze"], capture_output=True, text=True, check=True)
    return search_pip_freeze_output(project, r.stdout)


def run_stubgen(package: str, output: str) -> None:
    print(f"Running stubgen: stubgen -o {output} -p {package}")
    subprocess.run(["stubgen", "-o", output, "-p", package, "--export-less"], check=True)


def run_black(stub_dir: str) -> None:
    print(f"Running black: black {stub_dir}")
    subprocess.run(["black", stub_dir])


def run_isort(stub_dir: str) -> None:
    print(f"Running isort: isort {stub_dir}")
    subprocess.run(["python3", "-m", "isort", stub_dir])


def create_metadata(stub_dir: str, version: str) -> None:
    """Create a METADATA.toml file."""
    match = re.match(r"[0-9]+.[0-9]+", version)
    if match is None:
        sys.exit(f"Error: Cannot parse version number: {version}")
    filename = os.path.join(stub_dir, "METADATA.toml")
    version = match.group(0)
    if os.path.exists(filename):
        return
    print(f"Writing {filename}")
    with open(filename, "w", encoding="UTF-8") as file:
        file.write(
            f"""\
version = "{version}.*"

[tool.stubtest]
ignore_missing_stub = false
"""
        )


def add_pyright_exclusion(stub_dir: str) -> None:
    """Exclude stub_dir from strict pyright checks."""
    with open(PYRIGHT_CONFIG, encoding="UTF-8") as f:
        lines = f.readlines()
    i = 0
    while i < len(lines) and not lines[i].strip().startswith('"exclude": ['):
        i += 1
    assert i < len(lines), f"Error parsing {PYRIGHT_CONFIG}"
    while not lines[i].strip().startswith("]"):
        i += 1
    # Must use forward slash in the .json file
    line_to_add = f'        "{stub_dir}",'.replace("\\", "/")
    initial = i - 1
    while lines[i].lower() > line_to_add.lower():
        i -= 1
    if lines[i + 1].strip().rstrip(",") == line_to_add.strip().rstrip(","):
        print(f"{PYRIGHT_CONFIG} already up-to-date")
        return
    if i == initial:
        # Special case: when adding to the end of the list, commas need tweaking
        line_to_add = line_to_add.rstrip(",")
        lines[i] = lines[i].rstrip() + ",\n"
    lines.insert(i + 1, line_to_add + "\n")
    print(f"Updating {PYRIGHT_CONFIG}")
    with open(PYRIGHT_CONFIG, "w", encoding="UTF-8") as f:
        f.writelines(lines)


def main() -> None:
    parser = argparse.ArgumentParser(
        description="""Generate baseline stubs automatically for an installed pip package
                       using stubgen. Also run black and isort. If the name of
                       the project is different from the runtime Python package name, you may
                       need to use --package (example: --package yaml PyYAML)."""
    )
    parser.add_argument("project", help="name of PyPI project for which to generate stubs under stubs/")
    parser.add_argument("--package", help="generate stubs for this Python package (default is autodetected)")
    args = parser.parse_args()
    project = args.project
    package = args.package

    if not re.match(r"[a-zA-Z0-9-_.]+$", project):
        sys.exit(f"Invalid character in project name: {project!r}")

    if not package:
        package = project  # default
        # Try to find which packages are provided by the project
        # Use default if that fails or if several packages are found
        #
        # The importlib.metadata module is used for projects whose name is different
        # from the runtime Python package name (example: PyYAML/yaml)
        if sys.version_info >= (3, 8):
            dist = distribution(project).read_text("top_level.txt")
            if dist is not None:
                packages = [name for name in dist.split() if not name.startswith("_")]
                if len(packages) == 1:
                    package = packages[0]
        print(f'Using detected package "{package}" for project "{project}"', file=sys.stderr)
        print("Suggestion: Try again with --package argument if that's not what you wanted", file=sys.stderr)

    if not os.path.isdir("stubs") or not os.path.isdir("stdlib"):
        sys.exit("Error: Current working directory must be the root of typeshed repository")

    # Get normalized project name and version of installed package.
    info = get_installed_package_info(project)
    if info is None:
        print(f'Error: "{project}" is not installed', file=sys.stderr)
        print("", file=sys.stderr)
        print(f'Suggestion: Run "python3 -m pip install {project}" and try again', file=sys.stderr)
        sys.exit(1)
    project, version = info

    stub_dir = os.path.join("stubs", project)
    package_dir = os.path.join(stub_dir, package)
    if os.path.exists(package_dir):
        sys.exit(f"Error: {package_dir} already exists (delete it first)")

    run_stubgen(package, stub_dir)

    run_isort(stub_dir)
    run_black(stub_dir)

    create_metadata(stub_dir, version)

    # Since the generated stubs won't have many type annotations, we
    # have to exclude them from strict pyright checks.
    add_pyright_exclusion(stub_dir)

    print("\nDone!\n\nSuggested next steps:")
    print(f" 1. Manually review the generated stubs in {stub_dir}")
    print(" 2. Optionally run tests and autofixes (see tests/README.md for details)")
    print(" 3. Commit the changes on a new branch and create a typeshed PR (don't force-push!)")


if __name__ == "__main__":
    main()



from .data_dict import *

__all__ = ["get_emoji_unicode_dict", "get_aliases_unicode_dict", "EMOJI_DATA", "STATUS", "LANGUAGES"]

def get_emoji_unicode_dict(lang: str) -> dict[str, str]: ...
def get_aliases_unicode_dict() -> dict[str, str]: ...



from collections.abc import Iterator
from typing import Any, NoReturn

class JSONEncoder:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def encode(self, o: Any) -> str: ...
    def default(self, o: Any) -> NoReturn: ...
    def iterencode(self, o: Any, _one_shot: bool) -> Iterator[str]: ...

class JSONEncoderForHTML(JSONEncoder): ...



from _typeshed import ReadableBuffer
from collections.abc import Iterator
from typing import Any
from typing_extensions import TypeAlias

LATEST_MINOR_VERSION: int

class NodeVisitor:
    def visit(self, node: AST) -> Any: ...
    def generic_visit(self, node: AST) -> None: ...

class NodeTransformer(NodeVisitor):
    def generic_visit(self, node: AST) -> None: ...

def parse(
    source: str | ReadableBuffer, filename: str | ReadableBuffer = ..., mode: str = ..., feature_version: int = ...
) -> AST: ...
def copy_location(new_node: AST, old_node: AST) -> AST: ...
def dump(node: AST, annotate_fields: bool = ..., include_attributes: bool = ...) -> str: ...
def fix_missing_locations(node: AST) -> AST: ...
def get_docstring(node: AST, clean: bool = ...) -> str | None: ...
def increment_lineno(node: AST, n: int = ...) -> AST: ...
def iter_child_nodes(node: AST) -> Iterator[AST]: ...
def iter_fields(node: AST) -> Iterator[tuple[str, Any]]: ...
def literal_eval(node_or_string: str | AST) -> Any: ...
def walk(node: AST) -> Iterator[AST]: ...

PyCF_ONLY_AST: int

# ast classes

_Identifier: TypeAlias = str

class AST:
    _attributes: tuple[str, ...]
    _fields: tuple[str, ...]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class mod(AST): ...

class Module(mod):
    body: list[stmt]
    type_ignores: list[TypeIgnore]

class Interactive(mod):
    body: list[stmt]

class Expression(mod):
    body: expr

class FunctionType(mod):
    argtypes: list[expr]
    returns: expr

class Suite(mod):
    body: list[stmt]

class stmt(AST):
    lineno: int
    col_offset: int

class FunctionDef(stmt):
    name: _Identifier
    args: arguments
    body: list[stmt]
    decorator_list: list[expr]
    returns: expr | None
    type_comment: str | None

class AsyncFunctionDef(stmt):
    name: _Identifier
    args: arguments
    body: list[stmt]
    decorator_list: list[expr]
    returns: expr | None
    type_comment: str | None

class ClassDef(stmt):
    name: _Identifier
    bases: list[expr]
    keywords: list[keyword]
    body: list[stmt]
    decorator_list: list[expr]

class Return(stmt):
    value: expr | None

class Delete(stmt):
    targets: list[expr]

class Assign(stmt):
    targets: list[expr]
    value: expr
    type_comment: str | None

class AugAssign(stmt):
    target: expr
    op: operator
    value: expr

class AnnAssign(stmt):
    target: expr
    annotation: expr
    value: expr | None
    simple: int

class For(stmt):
    target: expr
    iter: expr
    body: list[stmt]
    orelse: list[stmt]
    type_comment: str | None

class AsyncFor(stmt):
    target: expr
    iter: expr
    body: list[stmt]
    orelse: list[stmt]
    type_comment: str | None

class While(stmt):
    test: expr
    body: list[stmt]
    orelse: list[stmt]

class If(stmt):
    test: expr
    body: list[stmt]
    orelse: list[stmt]

class With(stmt):
    items: list[withitem]
    body: list[stmt]
    type_comment: str | None

class AsyncWith(stmt):
    items: list[withitem]
    body: list[stmt]
    type_comment: str | None

class Raise(stmt):
    exc: expr | None
    cause: expr | None

class Try(stmt):
    body: list[stmt]
    handlers: list[ExceptHandler]
    orelse: list[stmt]
    finalbody: list[stmt]

class Assert(stmt):
    test: expr
    msg: expr | None

class Import(stmt):
    names: list[alias]

class ImportFrom(stmt):
    module: _Identifier | None
    names: list[alias]
    level: int | None

class Global(stmt):
    names: list[_Identifier]

class Nonlocal(stmt):
    names: list[_Identifier]

class Expr(stmt):
    value: expr

class Pass(stmt): ...
class Break(stmt): ...
class Continue(stmt): ...
class slice(AST): ...

_Slice: TypeAlias = slice  # this lets us type the variable named 'slice' below

class Slice(slice):
    lower: expr | None
    upper: expr | None
    step: expr | None

class ExtSlice(slice):
    dims: list[slice]

class Index(slice):
    value: expr

class expr(AST):
    lineno: int
    col_offset: int

class BoolOp(expr):
    op: boolop
    values: list[expr]

class BinOp(expr):
    left: expr
    op: operator
    right: expr

class UnaryOp(expr):
    op: unaryop
    operand: expr

class Lambda(expr):
    args: arguments
    body: expr

class IfExp(expr):
    test: expr
    body: expr
    orelse: expr

class Dict(expr):
    keys: list[expr]
    values: list[expr]

class Set(expr):
    elts: list[expr]

class ListComp(expr):
    elt: expr
    generators: list[comprehension]

class SetComp(expr):
    elt: expr
    generators: list[comprehension]

class DictComp(expr):
    key: expr
    value: expr
    generators: list[comprehension]

class GeneratorExp(expr):
    elt: expr
    generators: list[comprehension]

class Await(expr):
    value: expr

class Yield(expr):
    value: expr | None

class YieldFrom(expr):
    value: expr

class Compare(expr):
    left: expr
    ops: list[cmpop]
    comparators: list[expr]

class Call(expr):
    func: expr
    args: list[expr]
    keywords: list[keyword]

class Num(expr):
    n: complex

class Str(expr):
    s: str
    kind: str

class FormattedValue(expr):
    value: expr
    conversion: int | None
    format_spec: expr | None

class JoinedStr(expr):
    values: list[expr]

class Bytes(expr):
    s: bytes

class NameConstant(expr):
    value: Any

class Ellipsis(expr): ...

class Attribute(expr):
    value: expr
    attr: _Identifier
    ctx: expr_context

class Subscript(expr):
    value: expr
    slice: _Slice
    ctx: expr_context

class Starred(expr):
    value: expr
    ctx: expr_context

class Name(expr):
    id: _Identifier
    ctx: expr_context

class List(expr):
    elts: list[expr]
    ctx: expr_context

class Tuple(expr):
    elts: list[expr]
    ctx: expr_context

class expr_context(AST): ...
class AugLoad(expr_context): ...
class AugStore(expr_context): ...
class Del(expr_context): ...
class Load(expr_context): ...
class Param(expr_context): ...
class Store(expr_context): ...
class boolop(AST): ...
class And(boolop): ...
class Or(boolop): ...
class operator(AST): ...
class Add(operator): ...
class BitAnd(operator): ...
class BitOr(operator): ...
class BitXor(operator): ...
class Div(operator): ...
class FloorDiv(operator): ...
class LShift(operator): ...
class Mod(operator): ...
class Mult(operator): ...
class MatMult(operator): ...
class Pow(operator): ...
class RShift(operator): ...
class Sub(operator): ...
class unaryop(AST): ...
class Invert(unaryop): ...
class Not(unaryop): ...
class UAdd(unaryop): ...
class USub(unaryop): ...
class cmpop(AST): ...
class Eq(cmpop): ...
class Gt(cmpop): ...
class GtE(cmpop): ...
class In(cmpop): ...
class Is(cmpop): ...
class IsNot(cmpop): ...
class Lt(cmpop): ...
class LtE(cmpop): ...
class NotEq(cmpop): ...
class NotIn(cmpop): ...

class comprehension(AST):
    target: expr
    iter: expr
    ifs: list[expr]
    is_async: int

class ExceptHandler(AST):
    type: expr | None
    name: _Identifier | None
    body: list[stmt]
    lineno: int
    col_offset: int

class arguments(AST):
    args: list[arg]
    vararg: arg | None
    kwonlyargs: list[arg]
    kw_defaults: list[expr | None]
    kwarg: arg | None
    defaults: list[expr]

class arg(AST):
    arg: _Identifier
    annotation: expr | None
    lineno: int
    col_offset: int
    type_comment: str | None

class keyword(AST):
    arg: _Identifier | None
    value: expr

class alias(AST):
    name: _Identifier
    asname: _Identifier | None

class withitem(AST):
    context_expr: expr
    optional_vars: expr | None

class TypeIgnore(AST):
    lineno: int



import sys
from _typeshed import Incomplete, StrOrBytesPath
from collections.abc import Callable, Generator
from typing import NamedTuple, SupportsFloat, TypeVar, overload
from typing_extensions import Literal, ParamSpec, SupportsIndex, TypeAlias

from PIL import Image

_Unused: TypeAlias = object
_P = ParamSpec("_P")
_R = TypeVar("_R")
# TODO: cv2.Mat is not available as a type yet: https://github.com/microsoft/python-type-stubs/issues/211
# cv2.Mat is just an alias for a numpy NDArray, but can't import that either.
_Mat: TypeAlias = Incomplete

useOpenCV: bool
RUNNING_PYTHON_2 = sys.version_info < (3,)
GRAYSCALE_DEFAULT: Literal[False]
USE_IMAGE_NOT_FOUND_EXCEPTION: bool
scrotExists: bool

class Box(NamedTuple):
    left: int
    top: int
    width: int
    height: int

class Point(NamedTuple):
    x: int
    y: int

class RGB(NamedTuple):
    red: int
    green: int
    blue: int

class PyScreezeException(Exception): ...
class ImageNotFoundException(PyScreezeException): ...

def requiresPillow(wrappedFunction: Callable[_P, _R]) -> Callable[_P, _R]: ...
@overload
def locate(
    needleImage: str | Image.Image | _Mat,
    haystackImage: str | Image.Image | _Mat,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Box | None: ...

# _locateAll_python / _locateAll_pillow
@overload
def locate(
    needleImage: str | Image.Image,
    haystackImage: str | Image.Image,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: None = ...,
) -> Box | None: ...

# _locateAll_opencv
@overload
def locateOnScreen(
    image: str | Image.Image | _Mat,
    minSearchTime: float = ...,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Box | None: ...

# _locateAll_python / _locateAll_pillow
@overload
def locateOnScreen(
    image: str | Image.Image,
    minSearchTime: float = ...,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: None = ...,
) -> Box | None: ...

# _locateAll_opencv
@overload
def locateAllOnScreen(
    image: str | Image.Image | _Mat,
    *,
    grayscale: bool | None = ...,
    limit: int = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Generator[Box, None, None]: ...

# _locateAll_python / _locateAll_pillow
@overload
def locateAllOnScreen(
    image: str | Image.Image,
    *,
    grayscale: bool | None = ...,
    limit: int | None = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: None = ...,
) -> Generator[Box, None, None]: ...

# _locateAll_opencv
@overload
def locateCenterOnScreen(
    image: str | Image.Image | _Mat,
    *,
    minSearchTime: float,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Point | None: ...

# _locateAll_python / _locateAll_pillow
@overload
def locateCenterOnScreen(
    image: str | Image.Image,
    *,
    minSearchTime: float,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: None = ...,
) -> Point | None: ...

# _locateAll_opencv
@overload
def locateOnWindow(
    image: str | Image.Image | _Mat,
    title: str,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Box | None: ...

# _locateAll_python / _locateAll_pillow
@overload
def locateOnWindow(
    image: str | Image.Image,
    title: str,
    *,
    grayscale: bool | None = ...,
    limit: _Unused = ...,
    step: int = ...,
    confidence: None = ...,
) -> Box | None: ...
def showRegionOnScreen(region: tuple[int, int, int, int], outlineColor: str = ..., filename: str = ...) -> None: ...
def center(coords: tuple[int, int, int, int]) -> Point: ...
def pixelMatchesColor(
    x: int, y: int, expectedRGBColor: tuple[int, int, int] | tuple[int, int, int, int], tolerance: int = ...
) -> bool: ...
def pixel(x: int, y: int) -> tuple[int, int, int]: ...
def screenshot(imageFilename: StrOrBytesPath | None = ..., region: tuple[int, int, int, int] | None = ...) -> Image.Image: ...

grab = screenshot
# _locateAll_opencv
@overload
def locateAll(
    needleImage: str | Image.Image | _Mat,
    haystackImage: str | Image.Image | _Mat,
    grayscale: bool | None = ...,
    limit: int = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: SupportsFloat | SupportsIndex | str = ...,
) -> Generator[Box, None, None]: ...

# _locateAll_python / _locateAll_pillow
@overload
def locateAll(
    needleImage: str | Image.Image,
    haystackImage: str | Image.Image,
    grayscale: bool | None = ...,
    limit: int | None = ...,
    region: tuple[int, int, int, int] | None = ...,
    step: int = ...,
    confidence: None = ...,
) -> Generator[Box, None, None]: ...



from collections.abc import Callable, Iterable, Iterator
from sys import _OptExcInfo
from typing import Any, Protocol
from typing_extensions import TypeAlias

__all__ = ["StartResponse", "WSGIEnvironment", "WSGIApplication", "InputStream", "ErrorStream", "FileWrapper"]

class StartResponse(Protocol):
    def __call__(
        self, __status: str, __headers: list[tuple[str, str]], __exc_info: _OptExcInfo | None = ...
    ) -> Callable[[bytes], object]: ...

WSGIEnvironment: TypeAlias = dict[str, Any]
WSGIApplication: TypeAlias = Callable[[WSGIEnvironment, StartResponse], Iterable[bytes]]

class InputStream(Protocol):
    def read(self, __size: int = ...) -> bytes: ...
    def readline(self, __size: int = ...) -> bytes: ...
    def readlines(self, __hint: int = ...) -> list[bytes]: ...
    def __iter__(self) -> Iterator[bytes]: ...

class ErrorStream(Protocol):
    def flush(self) -> object: ...
    def write(self, __s: str) -> object: ...
    def writelines(self, __seq: list[str]) -> object: ...

class _Readable(Protocol):
    def read(self, __size: int = ...) -> bytes: ...
    # Optional: def close(self) -> object: ...

class FileWrapper(Protocol):
    def __call__(self, __file: _Readable, __block_size: int = ...) -> Iterable[bytes]: ...



# pyright: reportUnnecessaryTypeIgnoreComment=true

from invoke import Context, task

# ===========================================
# This snippet is a regression test for #8936
# ===========================================


@task
def docker_build(context: Context) -> None:
    pass


@task(docker_build)
def docker_push(context: Context) -> None:
    pass



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Drawing(Serialisable):
    tagname: str
    id: Any
    def __init__(self, id: Any | None = ...) -> None: ...



from _typeshed import Incomplete

class CffiOp:
    op: Incomplete
    arg: Incomplete
    def __init__(self, op, arg) -> None: ...
    def as_c_expr(self): ...
    def as_python_bytes(self): ...

def format_four_bytes(num): ...

OP_PRIMITIVE: int
OP_POINTER: int
OP_ARRAY: int
OP_OPEN_ARRAY: int
OP_STRUCT_UNION: int
OP_ENUM: int
OP_FUNCTION: int
OP_FUNCTION_END: int
OP_NOOP: int
OP_BITFIELD: int
OP_TYPENAME: int
OP_CPYTHON_BLTN_V: int
OP_CPYTHON_BLTN_N: int
OP_CPYTHON_BLTN_O: int
OP_CONSTANT: int
OP_CONSTANT_INT: int
OP_GLOBAL_VAR: int
OP_DLOPEN_FUNC: int
OP_DLOPEN_CONST: int
OP_GLOBAL_VAR_F: int
OP_EXTERN_PYTHON: int
PRIM_VOID: int
PRIM_BOOL: int
PRIM_CHAR: int
PRIM_SCHAR: int
PRIM_UCHAR: int
PRIM_SHORT: int
PRIM_USHORT: int
PRIM_INT: int
PRIM_UINT: int
PRIM_LONG: int
PRIM_ULONG: int
PRIM_LONGLONG: int
PRIM_ULONGLONG: int
PRIM_FLOAT: int
PRIM_DOUBLE: int
PRIM_LONGDOUBLE: int
PRIM_WCHAR: int
PRIM_INT8: int
PRIM_UINT8: int
PRIM_INT16: int
PRIM_UINT16: int
PRIM_INT32: int
PRIM_UINT32: int
PRIM_INT64: int
PRIM_UINT64: int
PRIM_INTPTR: int
PRIM_UINTPTR: int
PRIM_PTRDIFF: int
PRIM_SIZE: int
PRIM_SSIZE: int
PRIM_INT_LEAST8: int
PRIM_UINT_LEAST8: int
PRIM_INT_LEAST16: int
PRIM_UINT_LEAST16: int
PRIM_INT_LEAST32: int
PRIM_UINT_LEAST32: int
PRIM_INT_LEAST64: int
PRIM_UINT_LEAST64: int
PRIM_INT_FAST8: int
PRIM_UINT_FAST8: int
PRIM_INT_FAST16: int
PRIM_UINT_FAST16: int
PRIM_INT_FAST32: int
PRIM_UINT_FAST32: int
PRIM_INT_FAST64: int
PRIM_UINT_FAST64: int
PRIM_INTMAX: int
PRIM_UINTMAX: int
PRIM_FLOATCOMPLEX: int
PRIM_DOUBLECOMPLEX: int
PRIM_CHAR16: int
PRIM_CHAR32: int
PRIMITIVE_TO_INDEX: Incomplete
F_UNION: int
F_CHECK_FIELDS: int
F_PACKED: int
F_EXTERNAL: int
F_OPAQUE: int
G_FLAGS: Incomplete
CLASS_NAME: Incomplete



from typing import Any

from .langhelpers import (
    decorator as decorator,
    inject_docstring_text as inject_docstring_text,
    inject_param_text as inject_param_text,
)

SQLALCHEMY_WARN_20: bool

def warn_deprecated(msg, version, stacklevel: int = ..., code: Any | None = ...) -> None: ...
def warn_deprecated_limited(msg, args, version, stacklevel: int = ..., code: Any | None = ...) -> None: ...
def warn_deprecated_20(msg, stacklevel: int = ..., code: Any | None = ...) -> None: ...
def deprecated_cls(version, message, constructor: str = ...): ...
def deprecated_20_cls(clsname, alternative: Any | None = ..., constructor: str = ..., becomes_legacy: bool = ...): ...
def deprecated(
    version,
    message: Any | None = ...,
    add_deprecation_to_docstring: bool = ...,
    warning: Any | None = ...,
    enable_warnings: bool = ...,
): ...
def moved_20(message, **kw): ...
def deprecated_20(api_name, alternative: Any | None = ..., becomes_legacy: bool = ..., **kw): ...
def deprecated_params(**specs): ...



from typing import Any

from ..plugin import Error as Error, TConfig as TConfig, Visitor as Visitor

def assert_error(
    visitor_cls: type[Visitor[TConfig]], src: str, expected: type[Error], config: TConfig | None = ..., **kwargs: Any
) -> None: ...
def assert_not_error(visitor_cls: type[Visitor[TConfig]], src: str, config: TConfig | None = ...) -> None: ...



from _typeshed import Self
from collections.abc import Iterable
from re import Pattern
from typing import IO, Any

from paramiko.ssh_exception import ConfigParseError as ConfigParseError, CouldNotCanonicalize as CouldNotCanonicalize

SSH_PORT: int

class SSHConfig:
    SETTINGS_REGEX: Pattern[str]
    TOKENS_BY_CONFIG_KEY: dict[str, list[str]]
    def __init__(self) -> None: ...
    @classmethod
    def from_text(cls: type[Self], text: str) -> Self: ...
    @classmethod
    def from_path(cls: type[Self], path: str) -> Self: ...
    @classmethod
    def from_file(cls: type[Self], flo: IO[str]) -> Self: ...
    def parse(self, file_obj: IO[str]) -> None: ...
    def lookup(self, hostname: str) -> SSHConfigDict: ...
    def canonicalize(self, hostname: str, options: SSHConfigDict, domains: Iterable[str]) -> str: ...
    def get_hostnames(self) -> set[str]: ...

class LazyFqdn:
    fqdn: str | None
    config: SSHConfig
    host: str | None
    def __init__(self, config: SSHConfigDict, host: str | None = ...) -> None: ...

class SSHConfigDict(dict[str, str]):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def as_bool(self, key: str) -> bool: ...
    def as_int(self, key: str) -> int: ...



# pyright: reportWildcardImportFromLibrary=false

"""
This tests that star imports work when using "all += " syntax.
"""

import sys
from typing import *  # noqa: F403
from zipfile import *  # noqa: F403

if sys.version_info >= (3, 9):
    x: Annotated[int, 42]  # noqa: F405

if sys.version_info >= (3, 8):
    p: Path  # noqa: F405



from typing import Any

class BFInfo:
    capacity: Any
    size: Any
    filterNum: Any
    insertedNum: Any
    expansionRate: Any
    def __init__(self, args) -> None: ...

class CFInfo:
    size: Any
    bucketNum: Any
    filterNum: Any
    insertedNum: Any
    deletedNum: Any
    bucketSize: Any
    expansionRate: Any
    maxIteration: Any
    def __init__(self, args) -> None: ...

class CMSInfo:
    width: Any
    depth: Any
    count: Any
    def __init__(self, args) -> None: ...

class TopKInfo:
    k: Any
    width: Any
    depth: Any
    decay: Any
    def __init__(self, args) -> None: ...

class TDigestInfo:
    compression: Any
    capacity: Any
    mergedNodes: Any
    unmergedNodes: Any
    mergedWeight: Any
    unmergedWeight: Any
    totalCompressions: Any
    def __init__(self, args) -> None: ...



import sys
from _typeshed import OptExcInfo, ProfileFunction, TraceFunction, structseq
from builtins import object as _object
from collections.abc import AsyncGenerator, Callable, Coroutine, Sequence
from importlib.abc import PathEntryFinder
from importlib.machinery import ModuleSpec
from io import TextIOWrapper
from types import FrameType, ModuleType, TracebackType
from typing import Any, NoReturn, Protocol, TextIO, TypeVar, overload
from typing_extensions import Literal, TypeAlias, final

_T = TypeVar("_T")

# see https://github.com/python/typeshed/issues/8513#issue-1333671093 for the rationale behind this alias
_ExitCode: TypeAlias = str | int | None
_OptExcInfo: TypeAlias = OptExcInfo  # noqa: Y047  # TODO: obsolete, remove fall 2022 or later

# Intentionally omits one deprecated and one optional method of `importlib.abc.MetaPathFinder`
class _MetaPathFinder(Protocol):
    def find_spec(self, fullname: str, path: Sequence[str] | None, target: ModuleType | None = ...) -> ModuleSpec | None: ...

# ----- sys variables -----
if sys.platform != "win32":
    abiflags: str
argv: list[str]
base_exec_prefix: str
base_prefix: str
byteorder: Literal["little", "big"]
builtin_module_names: Sequence[str]  # actually a tuple of strings
copyright: str
if sys.platform == "win32":
    dllhandle: int
dont_write_bytecode: bool
displayhook: Callable[[object], Any]
excepthook: Callable[[type[BaseException], BaseException, TracebackType | None], Any]
exec_prefix: str
executable: str
float_repr_style: Literal["short", "legacy"]
hexversion: int
last_type: type[BaseException] | None
last_value: BaseException | None
last_traceback: TracebackType | None
maxsize: int
maxunicode: int
meta_path: list[_MetaPathFinder]
modules: dict[str, ModuleType]
if sys.version_info >= (3, 10):
    orig_argv: list[str]
path: list[str]
path_hooks: list[Callable[[str], PathEntryFinder]]
path_importer_cache: dict[str, PathEntryFinder | None]
platform: str
if sys.version_info >= (3, 9):
    platlibdir: str
prefix: str
if sys.version_info >= (3, 8):
    pycache_prefix: str | None
ps1: object
ps2: object
stdin: TextIO
stdout: TextIO
stderr: TextIO
if sys.version_info >= (3, 10):
    stdlib_module_names: frozenset[str]
__stdin__: TextIOWrapper
__stdout__: TextIOWrapper
__stderr__: TextIOWrapper
tracebacklimit: int
version: str
api_version: int
warnoptions: Any
#  Each entry is a tuple of the form (action, message, category, module,
#    lineno)
if sys.platform == "win32":
    winver: str
_xoptions: dict[Any, Any]

# Type alias used as a mixin for structseq classes that cannot be instantiated at runtime
# This can't be represented in the type system, so we just use `structseq[Any]`
_UninstantiableStructseq: TypeAlias = structseq[Any]

flags: _flags

if sys.version_info >= (3, 10):
    _FlagTuple: TypeAlias = tuple[int, int, int, int, int, int, int, int, int, int, int, int, int, bool, int, int]
else:
    _FlagTuple: TypeAlias = tuple[int, int, int, int, int, int, int, int, int, int, int, int, int, bool, int]

@final
class _flags(_UninstantiableStructseq, _FlagTuple):
    @property
    def debug(self) -> int: ...
    @property
    def inspect(self) -> int: ...
    @property
    def interactive(self) -> int: ...
    @property
    def optimize(self) -> int: ...
    @property
    def dont_write_bytecode(self) -> int: ...
    @property
    def no_user_site(self) -> int: ...
    @property
    def no_site(self) -> int: ...
    @property
    def ignore_environment(self) -> int: ...
    @property
    def verbose(self) -> int: ...
    @property
    def bytes_warning(self) -> int: ...
    @property
    def quiet(self) -> int: ...
    @property
    def hash_randomization(self) -> int: ...
    @property
    def isolated(self) -> int: ...
    @property
    def dev_mode(self) -> bool: ...
    @property
    def utf8_mode(self) -> int: ...
    if sys.version_info >= (3, 10):
        @property
        def warn_default_encoding(self) -> int: ...  # undocumented
    if sys.version_info >= (3, 11):
        @property
        def safe_path(self) -> bool: ...

float_info: _float_info

@final
class _float_info(structseq[float], tuple[float, int, int, float, int, int, int, int, float, int, int]):
    @property
    def max(self) -> float: ...  # DBL_MAX
    @property
    def max_exp(self) -> int: ...  # DBL_MAX_EXP
    @property
    def max_10_exp(self) -> int: ...  # DBL_MAX_10_EXP
    @property
    def min(self) -> float: ...  # DBL_MIN
    @property
    def min_exp(self) -> int: ...  # DBL_MIN_EXP
    @property
    def min_10_exp(self) -> int: ...  # DBL_MIN_10_EXP
    @property
    def dig(self) -> int: ...  # DBL_DIG
    @property
    def mant_dig(self) -> int: ...  # DBL_MANT_DIG
    @property
    def epsilon(self) -> float: ...  # DBL_EPSILON
    @property
    def radix(self) -> int: ...  # FLT_RADIX
    @property
    def rounds(self) -> int: ...  # FLT_ROUNDS

hash_info: _hash_info

@final
class _hash_info(structseq[Any | int], tuple[int, int, int, int, int, str, int, int, int]):
    @property
    def width(self) -> int: ...
    @property
    def modulus(self) -> int: ...
    @property
    def inf(self) -> int: ...
    @property
    def nan(self) -> int: ...
    @property
    def imag(self) -> int: ...
    @property
    def algorithm(self) -> str: ...
    @property
    def hash_bits(self) -> int: ...
    @property
    def seed_bits(self) -> int: ...
    @property
    def cutoff(self) -> int: ...  # undocumented

implementation: _implementation

class _implementation:
    name: str
    version: _version_info
    hexversion: int
    cache_tag: str
    # Define __getattr__, as the documentation states:
    # > sys.implementation may contain additional attributes specific to the Python implementation.
    # > These non-standard attributes must start with an underscore, and are not described here.
    def __getattr__(self, name: str) -> Any: ...

int_info: _int_info

@final
class _int_info(structseq[int], tuple[int, int, int, int]):
    @property
    def bits_per_digit(self) -> int: ...
    @property
    def sizeof_digit(self) -> int: ...
    @property
    def default_max_str_digits(self) -> int: ...
    @property
    def str_digits_check_threshold(self) -> int: ...

@final
class _version_info(_UninstantiableStructseq, tuple[int, int, int, str, int]):
    @property
    def major(self) -> int: ...
    @property
    def minor(self) -> int: ...
    @property
    def micro(self) -> int: ...
    @property
    def releaselevel(self) -> str: ...
    @property
    def serial(self) -> int: ...

version_info: _version_info

def call_tracing(__func: Callable[..., _T], __args: Any) -> _T: ...
def _clear_type_cache() -> None: ...
def _current_frames() -> dict[int, FrameType]: ...
def _getframe(__depth: int = ...) -> FrameType: ...
def _debugmallocstats() -> None: ...
def __displayhook__(__value: object) -> None: ...
def __excepthook__(__exctype: type[BaseException], __value: BaseException, __traceback: TracebackType | None) -> None: ...
def exc_info() -> OptExcInfo: ...

if sys.version_info >= (3, 11):
    def exception() -> BaseException | None: ...

def exit(__status: _ExitCode = ...) -> NoReturn: ...
def getallocatedblocks() -> int: ...
def getdefaultencoding() -> str: ...

if sys.platform != "win32":
    def getdlopenflags() -> int: ...

def getfilesystemencoding() -> str: ...
def getfilesystemencodeerrors() -> str: ...
def getrefcount(__object: Any) -> int: ...
def getrecursionlimit() -> int: ...
@overload
def getsizeof(obj: object) -> int: ...
@overload
def getsizeof(obj: object, default: int) -> int: ...
def getswitchinterval() -> float: ...
def getprofile() -> ProfileFunction | None: ...
def setprofile(profilefunc: ProfileFunction | None) -> None: ...
def gettrace() -> TraceFunction | None: ...
def settrace(tracefunc: TraceFunction | None) -> None: ...

if sys.platform == "win32":
    # A tuple of length 5, even though it has more than 5 attributes.
    @final
    class _WinVersion(_UninstantiableStructseq, tuple[int, int, int, int, str]):
        @property
        def major(self) -> int: ...
        @property
        def minor(self) -> int: ...
        @property
        def build(self) -> int: ...
        @property
        def platform(self) -> int: ...
        @property
        def service_pack(self) -> str: ...
        @property
        def service_pack_minor(self) -> int: ...
        @property
        def service_pack_major(self) -> int: ...
        @property
        def suite_mask(self) -> int: ...
        @property
        def product_type(self) -> int: ...
        @property
        def platform_version(self) -> tuple[int, int, int]: ...

    def getwindowsversion() -> _WinVersion: ...

def intern(__string: str) -> str: ...
def is_finalizing() -> bool: ...

__breakpointhook__: Any  # contains the original value of breakpointhook

def breakpointhook(*args: Any, **kwargs: Any) -> Any: ...

if sys.platform != "win32":
    def setdlopenflags(__flags: int) -> None: ...

def setrecursionlimit(__limit: int) -> None: ...
def setswitchinterval(__interval: float) -> None: ...
def gettotalrefcount() -> int: ...  # Debug builds only

if sys.version_info < (3, 9):
    def getcheckinterval() -> int: ...  # deprecated
    def setcheckinterval(__n: int) -> None: ...  # deprecated

if sys.version_info < (3, 9):
    # An 11-tuple or None
    def callstats() -> tuple[int, int, int, int, int, int, int, int, int, int, int] | None: ...

if sys.version_info >= (3, 8):
    # Doesn't exist at runtime, but exported in the stubs so pytest etc. can annotate their code more easily.
    class UnraisableHookArgs:
        exc_type: type[BaseException]
        exc_value: BaseException | None
        exc_traceback: TracebackType | None
        err_msg: str | None
        object: _object | None
    unraisablehook: Callable[[UnraisableHookArgs], Any]
    def __unraisablehook__(__unraisable: UnraisableHookArgs) -> Any: ...
    def addaudithook(hook: Callable[[str, tuple[Any, ...]], Any]) -> None: ...
    def audit(__event: str, *args: Any) -> None: ...

_AsyncgenHook: TypeAlias = Callable[[AsyncGenerator[Any, Any]], None] | None

@final
class _asyncgen_hooks(structseq[_AsyncgenHook], tuple[_AsyncgenHook, _AsyncgenHook]):
    @property
    def firstiter(self) -> _AsyncgenHook: ...
    @property
    def finalizer(self) -> _AsyncgenHook: ...

def get_asyncgen_hooks() -> _asyncgen_hooks: ...
def set_asyncgen_hooks(firstiter: _AsyncgenHook = ..., finalizer: _AsyncgenHook = ...) -> None: ...

if sys.platform == "win32":
    def _enablelegacywindowsfsencoding() -> None: ...

def get_coroutine_origin_tracking_depth() -> int: ...
def set_coroutine_origin_tracking_depth(depth: int) -> None: ...

if sys.version_info < (3, 8):
    _CoroWrapper: TypeAlias = Callable[[Coroutine[Any, Any, Any]], Any]
    def set_coroutine_wrapper(__wrapper: _CoroWrapper) -> None: ...
    def get_coroutine_wrapper() -> _CoroWrapper: ...

# The following two functions were added in 3.11.0, 3.10.7, 3.9.14, 3.8.14, & 3.7.14,
# as part of the response to CVE-2020-10735
def set_int_max_str_digits(maxdigits: int) -> None: ...
def get_int_max_str_digits() -> int: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class View3D(Serialisable):
    tagname: str
    rotX: Any
    x_rotation: Any
    hPercent: Any
    height_percent: Any
    rotY: Any
    y_rotation: Any
    depthPercent: Any
    rAngAx: Any
    right_angle_axes: Any
    perspective: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        rotX: int = ...,
        hPercent: Any | None = ...,
        rotY: int = ...,
        depthPercent: Any | None = ...,
        rAngAx: bool = ...,
        perspective: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class Surface(Serialisable):
    tagname: str
    thickness: Any
    spPr: Any
    graphicalProperties: Any
    pictureOptions: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self, thickness: Any | None = ..., spPr: Any | None = ..., pictureOptions: Any | None = ..., extLst: Any | None = ...
    ) -> None: ...

class _3DBase(Serialisable):
    tagname: str
    view3D: Any
    floor: Any
    sideWall: Any
    backWall: Any
    def __init__(
        self, view3D: Any | None = ..., floor: Any | None = ..., sideWall: Any | None = ..., backWall: Any | None = ...
    ) -> None: ...



from collections.abc import Generator
from typing import Any

CELL_TAG: Any
VALUE_TAG: Any
FORMULA_TAG: Any
MERGE_TAG: Any
INLINE_STRING: Any
COL_TAG: Any
ROW_TAG: Any
CF_TAG: Any
LEGACY_TAG: Any
PROT_TAG: Any
EXT_TAG: Any
HYPERLINK_TAG: Any
TABLE_TAG: Any
PRINT_TAG: Any
MARGINS_TAG: Any
PAGE_TAG: Any
HEADER_TAG: Any
FILTER_TAG: Any
VALIDATION_TAG: Any
PROPERTIES_TAG: Any
VIEWS_TAG: Any
FORMAT_TAG: Any
ROW_BREAK_TAG: Any
COL_BREAK_TAG: Any
SCENARIOS_TAG: Any
DATA_TAG: Any
DIMENSION_TAG: Any
CUSTOM_VIEWS_TAG: Any

class WorkSheetParser:
    min_row: Any
    epoch: Any
    source: Any
    shared_strings: Any
    data_only: Any
    shared_formulae: Any
    array_formulae: Any
    row_counter: int
    tables: Any
    date_formats: Any
    timedelta_formats: Any
    row_dimensions: Any
    column_dimensions: Any
    number_formats: Any
    keep_vba: bool
    hyperlinks: Any
    formatting: Any
    legacy_drawing: Any
    merged_cells: Any
    row_breaks: Any
    col_breaks: Any
    def __init__(
        self, src, shared_strings, data_only: bool = ..., epoch=..., date_formats=..., timedelta_formats=...
    ) -> None: ...
    def parse(self) -> Generator[Any, None, None]: ...
    def parse_dimensions(self): ...
    col_counter: Any
    def parse_cell(self, element): ...
    def parse_formula(self, element): ...
    def parse_column_dimensions(self, col) -> None: ...
    def parse_row(self, row): ...
    def parse_formatting(self, element) -> None: ...
    protection: Any
    def parse_sheet_protection(self, element) -> None: ...
    def parse_extensions(self, element) -> None: ...
    def parse_legacy(self, element) -> None: ...
    def parse_row_breaks(self, element) -> None: ...
    def parse_col_breaks(self, element) -> None: ...
    def parse_custom_views(self, element) -> None: ...

class WorksheetReader:
    ws: Any
    parser: Any
    tables: Any
    def __init__(self, ws, xml_source, shared_strings, data_only) -> None: ...
    def bind_cells(self) -> None: ...
    def bind_formatting(self) -> None: ...
    def bind_tables(self) -> None: ...
    def bind_merged_cells(self) -> None: ...
    def bind_hyperlinks(self) -> None: ...
    def normalize_merged_cell_link(self, coord): ...
    def bind_col_dimensions(self) -> None: ...
    def bind_row_dimensions(self) -> None: ...
    def bind_properties(self) -> None: ...
    def bind_all(self) -> None: ...



from stripe.api_resources.identity.verification_report import VerificationReport as VerificationReport
from stripe.api_resources.identity.verification_session import VerificationSession as VerificationSession



from _typeshed import Incomplete
from datetime import datetime
from enum import Enum
from typing import Any

from ..vmodl.query import PropertyCollector
from .event import EventManager
from .option import OptionManager
from .view import ViewManager

def __getattr__(name: str) -> Incomplete: ...

class ManagedObject: ...

class ManagedEntity(ManagedObject):
    _moId: str
    obj: None
    name: str
    def __getattr__(self, name: str) -> Incomplete: ...

class ServiceInstanceContent:
    setting: OptionManager
    propertyCollector: PropertyCollector
    rootFolder: Folder
    viewManager: ViewManager
    perfManager: PerformanceManager
    eventManager: EventManager
    def __getattr__(self, name: str) -> Incomplete: ...

class ServiceInstance:
    content: ServiceInstanceContent
    def CurrentTime(self) -> datetime: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class PerformanceManager:
    class MetricId:
        counterId: int
        instance: str
        def __init__(self, counterId: int, instance: str) -> None: ...

    class PerfCounterInfo:
        key: int
        groupInfo: Any
        nameInfo: Any
        rollupType: Any
        def __getattr__(self, name: str) -> Incomplete: ...

    class QuerySpec:
        entity: ManagedEntity
        metricId: list[PerformanceManager.MetricId]
        intervalId: int
        maxSample: int
        startTime: datetime
        def __getattr__(self, name: str) -> Incomplete: ...

    class EntityMetricBase:
        entity: ManagedEntity
    def QueryPerfCounterByLevel(self, collection_level: int) -> list[PerformanceManager.PerfCounterInfo]: ...
    def QueryPerf(self, querySpec: list[PerformanceManager.QuerySpec]) -> list[PerformanceManager.EntityMetricBase]: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class ClusterComputeResource(ManagedEntity): ...
class ComputeResource(ManagedEntity): ...
class Datacenter(ManagedEntity): ...
class Datastore(ManagedEntity): ...
class Folder(ManagedEntity): ...
class HostSystem(ManagedEntity): ...
class VirtualMachine(ManagedEntity): ...

class VirtualMachinePowerState(Enum):
    poweredOff: int
    poweredOn: int
    suspended: int



from contextlib import AbstractContextManager

from .config import Config, DataProxy

class Context(DataProxy):
    def __init__(self, config: Config | None = ...) -> None: ...
    @property
    def config(self) -> Config: ...
    @config.setter
    def config(self, value: Config) -> None: ...
    def run(self, command: str, **kwargs): ...
    def sudo(self, command: str, *, password: str = ..., user: str = ..., **kwargs): ...
    def prefix(self, command: str) -> AbstractContextManager[None]: ...
    @property
    def cwd(self) -> str: ...
    def cd(self, path: str) -> AbstractContextManager[None]: ...

class MockContext(Context):
    def __init__(self, config: Config | None = ..., **kwargs) -> None: ...
    def run(self, command: str, *args, **kwargs): ...
    def sudo(self, command: str, *args, **kwargs): ...
    def set_result_for(self, attname, command, result) -> None: ...



from typing import Any

from .base import DATE, DATETIME, SQLiteDialect

class _SQLite_pysqliteTimeStamp(DATETIME):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class _SQLite_pysqliteDate(DATE):
    def bind_processor(self, dialect): ...
    def result_processor(self, dialect, coltype): ...

class SQLiteDialect_pysqlite(SQLiteDialect):
    default_paramstyle: str
    supports_statement_cache: bool
    colspecs: Any
    description_encoding: Any
    driver: str
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url): ...
    def set_isolation_level(self, connection, level): ...
    def on_connect(self): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...

dialect = SQLiteDialect_pysqlite



__all__ = ["body_decode", "body_encode", "decode", "decodestring", "header_encode", "header_length"]

from _typeshed import ReadableBuffer

def header_length(bytearray: str | bytes | bytearray) -> int: ...
def header_encode(header_bytes: str | ReadableBuffer, charset: str = ...) -> str: ...

# First argument should be a buffer that supports slicing and len().
def body_encode(s: bytes | bytearray, maxlinelen: int = ..., eol: str = ...) -> str: ...
def decode(string: str | ReadableBuffer) -> bytes: ...

body_decode = decode
decodestring = decode



from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

MAGIC: bytes
FORMAT_DXT1: int
FORMAT_UNCOMPRESSED: int

class FtexImageFile(ImageFile):
    format: ClassVar[Literal["FTEX"]]
    format_description: ClassVar[str]
    def load_seek(self, pos) -> None: ...



import threading
from _typeshed import Self
from collections.abc import Awaitable
from types import SimpleNamespace, TracebackType
from typing import Any, ClassVar

from redis.asyncio import Redis
from redis.commands.core import AsyncScript

class Lock:
    lua_release: ClassVar[AsyncScript | None]
    lua_extend: ClassVar[AsyncScript | None]
    lua_reacquire: ClassVar[AsyncScript | None]
    LUA_RELEASE_SCRIPT: ClassVar[str]
    LUA_EXTEND_SCRIPT: ClassVar[str]
    LUA_REACQUIRE_SCRIPT: ClassVar[str]
    redis: Redis[Any]
    name: str | bytes | memoryview
    timeout: float | None
    sleep: float
    blocking: bool
    blocking_timeout: float | None
    thread_local: bool
    local: threading.local | SimpleNamespace
    def __init__(
        self,
        redis: Redis[Any],
        name: str | bytes | memoryview,
        timeout: float | None = ...,
        sleep: float = ...,
        blocking: bool = ...,
        blocking_timeout: float | None = ...,
        thread_local: bool = ...,
    ) -> None: ...
    def register_scripts(self) -> None: ...
    async def __aenter__(self: Self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    async def acquire(
        self, blocking: bool | None = ..., blocking_timeout: float | None = ..., token: str | bytes | None = ...
    ) -> bool: ...
    async def do_acquire(self, token: str | bytes) -> bool: ...
    async def locked(self) -> bool: ...
    async def owned(self) -> bool: ...
    def release(self) -> Awaitable[None]: ...
    async def do_release(self, expected_token: bytes) -> None: ...
    def extend(self, additional_time: float, replace_ttl: bool = ...) -> Awaitable[bool]: ...
    async def do_extend(self, additional_time: float, replace_ttl: bool) -> bool: ...
    def reacquire(self) -> Awaitable[bool]: ...
    async def do_reacquire(self) -> bool: ...
